@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -11,7 +11,9 @@
 #ifndef MY_MIME_BASE64
 #define MY_MIME_BASE64
 
-void three_to_four(unsigned char *what,unsigned char *where);
-char * string_to_base64( char *what);
+#include <string>
+
+void three_to_four(const unsigned char *what,unsigned char *where);
+char * string_to_base64(const char *what);
 
 #endif
