@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -9,6 +9,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
+
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -28,8 +29,6 @@
 
 void get_size_of_clist();
 
-int TEMP_COLUMN_SIZES[NOTHING_COL+1];
-int TEMP_COLUMN_POS[NOTHING_COL+1];
 tOption downloader_parsed_args[]={
 	{"--help",		OPT_HELP},
 	{"-h",			OPT_HELP},
@@ -58,10 +57,14 @@ tOption downloader_parsed_args[]={
 	{"--minimized",		OPT_RUN_MINIMIZED},
 	{"--exit-time",		OPT_EXIT_TIME},
 	{"--ls",		OPT_LS},
+	{"--switch",		OPT_SWITCH},
 	{"--del",		OPT_DEL},
 	{"--stop",		OPT_STOP},
 	{"--color",		OPT_COLOR},
-	{"-geometry",		OPT_GEOMETRY}
+	{"--referer",		OPT_REFERER},
+	{"-geometry",		OPT_GEOMETRY},
+	{"-a",			OPT_ADD_OPEN},
+	{"--al",		OPT_ADD_OPENLIST}
 };
 
 char *downloader_args_errors[]={
@@ -85,26 +88,26 @@ char *downloader_args_errors[]={
 	(char *)N_("Expect URL as parameter for '--del' option"),
 	(char *)N_("Expect string in format WIDTHxHEIGHT+X+Y as parameter for '-geometry'"),
 	(char *)N_("Expect URL as parameter for '--stop' option"),
-	(char *)NULL
+	(char *)NULL,
+	(char *)N_("Expect URL as parameter for '--referer' option"),
+	(char *)NULL,
+	(char *)N_("Expect URL as parameter for '-a' option"),
+	(char *)N_("Expect at least one URL as parameter for '--al' option")
 };
 
 
 tConfigVariable config_variables[]={
-	{"max_threads",		CV_TYPE_INT,	&(CFG.MAX_THREADS)},
 	{"max_log",		CV_TYPE_INT,	&(CFG.MAX_LOG_LENGTH)},
 	{"max_main_log",	CV_TYPE_INT,	&(CFG.MAX_MAIN_LOG_LENGTH)},
 	{"max_retries",		CV_TYPE_INT,	&(CFG.DEFAULT_CFG.number_of_attempts)},
 	{"timeout",		CV_TYPE_INT,	&(CFG.DEFAULT_CFG.timeout)},
 	{"break_timeout",	CV_TYPE_INT,	&(CFG.DEFAULT_CFG.time_for_sleep)},
 	{"optimize",		CV_TYPE_BOOL,	&(CFG.RECURSIVE_OPTIMIZE)},
-	{"del_completed",	CV_TYPE_BOOL,	&(CFG.DELETE_COMPLETED)},
-	{"del_fataled",		CV_TYPE_BOOL,	&(CFG.DELETE_FATAL)},
-	{"nice_decs",		CV_TYPE_INT,	&(CFG.NICE_DEC_DIGITALS.curent)},
-	{"time_format",		CV_TYPE_INT,	&(CFG.TIME_FORMAT)},
 	{"ftp_passive_mode",	CV_TYPE_BOOL,	&(CFG.DEFAULT_CFG.passive)},
 	{"retry_if_noreget",	CV_TYPE_BOOL,	&(CFG.DEFAULT_CFG.retry)},
 	{"ftp_dirontop",	CV_TYPE_BOOL,	&(CFG.DEFAULT_CFG.ftp_dirontop)},
 	{"sleeptime",		CV_TYPE_INT,	&(CFG.DEFAULT_CFG.time_for_sleep)},
+	{"ihate_etag",		CV_TYPE_BOOL,	&(CFG.DEFAULT_CFG.ihate_etag)},
 	{"savepath",		CV_TYPE_STRING,	&(CFG.GLOBAL_SAVE_PATH)},
 	{"xposition",		CV_TYPE_INT,	&(CFG.WINDOW_X_POSITION)},
 	{"yposition",		CV_TYPE_INT,	&(CFG.WINDOW_Y_POSITION)},
@@ -112,6 +115,7 @@ tConfigVariable config_variables[]={
 	{"windowheight",	CV_TYPE_INT,	&(CFG.WINDOW_HEIGHT)},
 	{"clist_height",	CV_TYPE_INT,	&(CFG.WINDOW_CLIST_HEIGHT)},
 	{"clist_width",		CV_TYPE_INT,	&(CFG.WINDOW_CLIST_WIDTH)},
+	{"tree_width",		CV_TYPE_INT,	&(CFG.WINDOW_TREE_WIDTH)},
 	{"ftp_proxy_host",	CV_TYPE_STRING,	&(CFG.FTP_PROXY_HOST)},
 	{"ftp_proxy_user",	CV_TYPE_STRING,	&(CFG.FTP_PROXY_USER)},
 	{"ftp_proxy_pass",	CV_TYPE_STRING,	&(CFG.FTP_PROXY_PASS)},
@@ -133,24 +137,9 @@ tConfigVariable config_variables[]={
 	{"save_list",		CV_TYPE_BOOL,	&(CFG.SAVE_LIST)},
 	{"interval_save_list",	CV_TYPE_INT,	&(CFG.SAVE_LIST_INTERVAL)},
 	{"use_mainwin_title",	CV_TYPE_BOOL,	&(CFG.USE_MAINWIN_TITLE)},
-	{"use_mainwin_titleII",	CV_TYPE_BOOL,	&(CFG.USE_MAINWIN_TITLE2)},
+	{"use_mainwin_titleII",	CV_TYPE_INT,	&(CFG.USE_MAINWIN_TITLE2)},
 	{"get_date_from_server",CV_TYPE_BOOL,	&(CFG.DEFAULT_CFG.get_date)},
 	{"need_dialog_for_dnd",	CV_TYPE_BOOL,	&(CFG.NEED_DIALOG_FOR_DND)},
-	{"dl_status_col",	CV_TYPE_INT,	&(TEMP_COLUMN_SIZES[STATUS_COL])},
-	{"dl_file_col",		CV_TYPE_INT,	&(TEMP_COLUMN_SIZES[FILE_COL])},
-	{"dl_file_type_col",	CV_TYPE_INT,	&(TEMP_COLUMN_SIZES[FILE_TYPE_COL])},
-	{"dl_full_size_col",	CV_TYPE_INT,	&(TEMP_COLUMN_SIZES[FULL_SIZE_COL])},
-	{"dl_downloaded_size_col",CV_TYPE_INT,	&(TEMP_COLUMN_SIZES[DOWNLOADED_SIZE_COL])},
-	{"dl_remain_size_col",	CV_TYPE_INT,	&(TEMP_COLUMN_SIZES[REMAIN_SIZE_COL])},
-	{"dl_percent_col",	CV_TYPE_INT,	&(TEMP_COLUMN_SIZES[PERCENT_COL])},
-	{"dl_speed_col",	CV_TYPE_INT,	&(TEMP_COLUMN_SIZES[SPEED_COL])},
-	{"dl_time_col",		CV_TYPE_INT,	&(TEMP_COLUMN_SIZES[TIME_COL])},
-	{"dl_elapsed_time_col",	CV_TYPE_INT,	&(TEMP_COLUMN_SIZES[ELAPSED_TIME_COL])},
-	{"dl_pause_col",	CV_TYPE_INT,	&(TEMP_COLUMN_SIZES[PAUSE_COL])},
-	{"dl_treat_col",	CV_TYPE_INT,	&(TEMP_COLUMN_SIZES[TREAT_COL])},
-	{"dl_desc_col",		CV_TYPE_INT,	&(TEMP_COLUMN_SIZES[DESCRIPTION_COL])},
-	{"fl_host_col",		CV_TYPE_INT,	&(CFG.FACE_LIMITS_SIZE1)},
-	{"fl_limit_col",	CV_TYPE_INT,	&(CFG.FACE_LIMITS_SIZE2)},
 	{"window_lower",	CV_TYPE_BOOL,	&(CFG.WINDOW_LOWER)},
 	{"confirm_exit",	CV_TYPE_BOOL,	&(CFG.CONFIRM_EXIT)},
 	{"confirm_delete",	CV_TYPE_BOOL,	&(CFG.CONFIRM_DELETE)},
@@ -167,21 +156,6 @@ tConfigVariable config_variables[]={
 	{"default_name",	CV_TYPE_STRING,	&(CFG.DEFAULT_NAME)},
 	{"scroll_mainwin_title",CV_TYPE_BOOL,	&(CFG.SCROLL_MAINWIN_TITLE)},
 	{"default_permisions",	CV_TYPE_INT,	&(CFG.DEFAULT_PERMISIONS)},
-	{"dl_status_col_pos",	CV_TYPE_INT,	&(TEMP_COLUMN_POS[STATUS_COL])},
-	{"dl_file_col_pos",	CV_TYPE_INT,	&(TEMP_COLUMN_POS[FILE_COL])},
-	{"dl_file_type_col_pos",	CV_TYPE_INT,	&(TEMP_COLUMN_POS[FILE_TYPE_COL])},
-	{"dl_full_size_col_pos",	CV_TYPE_INT,	&(TEMP_COLUMN_POS[FULL_SIZE_COL])},
-	{"dl_downloaded_size_col_pos",	CV_TYPE_INT,	&(TEMP_COLUMN_POS[DOWNLOADED_SIZE_COL])},
-	{"dl_remain_size_col_pos",	CV_TYPE_INT,	&(TEMP_COLUMN_POS[REMAIN_SIZE_COL])},
-	{"dl_percent_col_pos",	CV_TYPE_INT,	&(TEMP_COLUMN_POS[PERCENT_COL])},
-	{"dl_speed_col_pos",	CV_TYPE_INT,	&(TEMP_COLUMN_POS[SPEED_COL])},
-	{"dl_time_col_pos",	CV_TYPE_INT,	&(TEMP_COLUMN_POS[TIME_COL])},
-	{"dl_elapsed_time_col_pos",	CV_TYPE_INT,	&(TEMP_COLUMN_POS[ELAPSED_TIME_COL])},
-	{"dl_pause_col_pos",	CV_TYPE_INT,	&(TEMP_COLUMN_POS[PAUSE_COL])},
-	{"dl_treat_col_pos",	CV_TYPE_INT,	&(TEMP_COLUMN_POS[TREAT_COL])},
-	{"dl_desc_col_pos",	CV_TYPE_INT,	&(TEMP_COLUMN_POS[DESCRIPTION_COL])},
-	{"dl_url_col_pos",	CV_TYPE_INT,	&(TEMP_COLUMN_POS[URL_COL])},
-	{"dl_nothing_col_pos",	CV_TYPE_INT,	&(TEMP_COLUMN_POS[NOTHING_COL])},
 	{"rollback",		CV_TYPE_INT,	&(CFG.DEFAULT_CFG.rollback)},
 	{"dnd_trash",		CV_TYPE_BOOL,	&(CFG.DND_TRASH)},
 	{"dnd_trash_x",		CV_TYPE_INT,	&(CFG.DND_TRASH_X)},
@@ -194,6 +168,7 @@ tConfigVariable config_variables[]={
 	{"graph_fore1",		CV_TYPE_HEX,	&(CFG.GRAPH_FORE1)},
 	{"graph_fore2",		CV_TYPE_HEX,	&(CFG.GRAPH_FORE2)},
 	{"graph_pick",		CV_TYPE_HEX,	&(CFG.GRAPH_PICK)},
+	{"graph_mode",		CV_TYPE_INT,	&(CFG.GRAPH_MODE)},
 	{"exec_when_quit",	CV_TYPE_STRING,	&(CFG.EXEC_WHEN_QUIT)},
 	{"remember_pass",	CV_TYPE_BOOL,	&(CFG.REMEMBER_PASS)},
 	{"clipboard_monitor",	CV_TYPE_BOOL,	&(CFG.CLIPBOARD_MONITOR)},
@@ -203,11 +178,10 @@ tConfigVariable config_variables[]={
 	{"buttons_flags",	CV_TYPE_HEX,	&(CFG.BUTTONS_FLAGS)},
 	{"main_log_file_limit",	CV_TYPE_LONG,	&(CFG.MAIN_LOG_FILE_LIMIT)},
 	{"fixed_log_font",	CV_TYPE_BOOL,	&(CFG.FIXED_LOG_FONT)},
-	{"default_host_limit",	CV_TYPE_INT,	&(CFG.DEFAULT_HOST_LIMIT)},
 	{"allow_force_run",	CV_TYPE_BOOL,	&(CFG.ALLOW_FORCE_RUN)},
 	{"ftp_dir_in_log",	CV_TYPE_BOOL,	&(CFG.FTP_DIR_IN_LOG)},
 	{"dont_send_quit",	CV_TYPE_BOOL,	&(CFG.DEFAULT_CFG.dont_send_quit)},
-	{"link_as_file",	CV_TYPE_BOOL,	&(CFG.DEFAULT_CFG.link_as_file)},
+	{"follow_link",		CV_TYPE_BOOL,	&(CFG.DEFAULT_CFG.follow_link)},
 	{"http_leave_server",	CV_TYPE_BOOL,	&(CFG.DEFAULT_CFG.leave_server)},
 	{"http_leave_dir",	CV_TYPE_BOOL,	&(CFG.DEFAULT_CFG.dont_leave_dir)},
 	{"sleep_before_complete",	CV_TYPE_BOOL,	&(CFG.DEFAULT_CFG.sleep_before_complete)},
@@ -215,7 +189,9 @@ tConfigVariable config_variables[]={
 	{"check_time",		CV_TYPE_BOOL,	&(CFG.DEFAULT_CFG.check_time)},
 	{"pause_after_adding",	CV_TYPE_BOOL,	&(CFG.PAUSE_AFTER_ADDING)},
 	{"search_ping_times",	CV_TYPE_INT,	&(CFG.SEARCH_PING_TIMES)},
-	{"search_host",		CV_TYPE_INT,	&(CFG.SEARCH_HOST)},
+	{"search_engines",	CV_TYPE_STRING,	&(CFG.SEARCH_ENGINES)},
+	{"search_perserver",    CV_TYPE_INT,	&(CFG.SEARCH_PERSERVER)},
+//	{"search_host",		CV_TYPE_INT,	&(CFG.SEARCH_HOST)},
 	{"search_entries",	CV_TYPE_INT,	&(CFG.SEARCH_ENTRIES)},
 	{"change_links",	CV_TYPE_BOOL,	&(CFG.DEFAULT_CFG.change_links)},
 	{"anonymous_pass",	CV_TYPE_STRING,	&(CFG.ANONYMOUS_PASS)},
@@ -225,6 +201,7 @@ tConfigVariable config_variables[]={
 	{"socks_port",		CV_TYPE_INT,	&(CFG.SOCKS_PORT)},
 	{"progress_mode",	CV_TYPE_INT,	&(CFG.PROGRESS_MODE)},
 	{"enable_sounds",	CV_TYPE_BOOL,	&(CFG.ENABLE_SOUNDS)},
+	{"esd_sound",		CV_TYPE_BOOL,	&(CFG.ESD_SOUND)},
 	{"sound_complete",	CV_TYPE_STRING,	&(CFG.SOUND_COMPLETE)},
 	{"sound_fail",		CV_TYPE_STRING,	&(CFG.SOUND_FAIL)},
 	{"sound_add",		CV_TYPE_STRING,	&(CFG.SOUND_ADD)},
@@ -233,18 +210,20 @@ tConfigVariable config_variables[]={
 	{"sound_startup",	CV_TYPE_STRING,	&(CFG.SOUND_STARTUP)},
 	{"clist_shift",		CV_TYPE_FLOAT,	&(CFG.CLIST_SHIFT)},
 	{"default_filter",	CV_TYPE_STRING,	&(CFG.DEFAULT_FILTER)},
-	{"donotset_winpos",	CV_TYPE_BOOL,	&(CFG.DONOTSET_WINPOS)}
+	{"donotset_winpos",	CV_TYPE_BOOL,	&(CFG.DONOTSET_WINPOS)},
+	{"hide_main_window",	CV_TYPE_BOOL,	&(CFG.HIDE_MAIN_WINDOW)},
+	{"theme_file",		CV_TYPE_STRING,	&(CFG.THEME_FILE)},
+	{"use_theme",		CV_TYPE_BOOL,	&(CFG.USE_THEME)},
+	{"themes_dir",		CV_TYPE_STRING,	&(CFG.THEMES_DIR)},
+	{"use_default_cfg",	CV_TYPE_BOOL,	&(CFG.USE_DEFAULT_CFG)},
+	{"number_of_parts",	CV_TYPE_INT,	&(CFG.NUMBER_OF_PARTS)},
+	{"graph_on_basket",	CV_TYPE_INT,	&(CFG.GRAPH_ON_BASKET)},
+	{"speed_on_basket",	CV_TYPE_INT,	&(CFG.SHOW_SPEED_ON_BASKET)},
+	{"dblclk_act",		CV_TYPE_INT,	&(CFG.DBLCLK_ACT)}
 };
 
 int downloader_parsed_args_num=sizeof(downloader_parsed_args)/sizeof(tOption);
 
-void set_column_position(int type,int col){
-	if (col>=0 && col<=NOTHING_COL){
-		ListColumns[col].type=type;
-		ListColumns[type].enum_index=col;
-	};
-};
-
 void set_config(char *line){
 	DBC_RETURN_IF_FAIL(line!=NULL);
 	if(*line=='\n' || *line=='#') return;
@@ -298,45 +277,36 @@ void read_config() {
 	char *cfgpath=compose_path(HOME_VARIABLE,CFG_FILE);
 	int fd=open(cfgpath,O_RDONLY);
 	if (fd>=0) {
-		for(int i=0;i<=NOTHING_COL;i++){
-			TEMP_COLUMN_POS[i]=i;
-			TEMP_COLUMN_SIZES[i]=ListColumns[ListColumns[i].enum_index].size;
-		};
 		char temp[MAX_LEN];
-		init_columns_info();
 		while(f_rstr(fd,temp,MAX_LEN)) {
 			set_config(temp);
 		};
 		close(fd);
-		for(int i=0;i<=NOTHING_COL;i++){
-			set_column_position(i,TEMP_COLUMN_POS[i]);
-		}
-		for(int i=0;i<NOTHING_COL;i++)
-			ListColumns[ListColumns[i].enum_index].size=TEMP_COLUMN_SIZES[i];
 	} else {
 		save_config();
-		printf(_("Can't open cfg file at '%s'\n"),cfgpath);
-		printf(_("Use default cfg :))...\n"));
+		g_print(_("Can't open cfg file at '%s'\n"),cfgpath);
+		g_print(_("Use default cfg :))...\n"));
 	};
 	delete[] cfgpath;
-	load_strlist(ALL_HISTORIES[URL_HISTORY], ".ntrc/history1",0);
-	load_strlist(ALL_HISTORIES[PATH_HISTORY],".ntrc/history2",1);
-	load_strlist(ALL_HISTORIES[LOG_HISTORY],".ntrc/history3",0);
-	load_strlist(ALL_HISTORIES[LOAD_SAVE_HISTORY],".ntrc/history4",1);
-	load_strlist(ALL_HISTORIES[USER_HISTORY],".ntrc/history5",0);
-	load_strlist(ALL_HISTORIES[PROXY_HISTORY],".ntrc/history6",0);
-	load_strlist(ALL_HISTORIES[FILE_HISTORY],".ntrc/history7",0);
-	load_strlist(ALL_HISTORIES[USER_AGENT_HISTORY],".ntrc/history8",0);
-	load_strlist(ALL_HISTORIES[EXEC_HISTORY],".ntrc/history9",0);
-	load_strlist(ALL_HISTORIES[SKIP_HISTORY],".ntrc/history11",0);
-	load_strlist(ALL_HISTORIES[SAVE_HISTORY],".ntrc/history12",1);
-	load_strlist(ALL_HISTORIES[LOG_SAVE_HISTORY],".ntrc/history13",1);
-	load_strlist(ALL_HISTORIES[DESC_HISTORY],".ntrc/history14",0);
-	load_strlist(ALL_HISTORIES[REFERER_HISTORY],".ntrc/history15",0);
-	load_strlist(ALL_HISTORIES[COOKIE_HISTORY],".ntrc/history16",0);
-	load_strlist(ALL_HISTORIES[SOUNDS_HISTORY],".ntrc/history17",1);
+	load_strlist(ALL_HISTORIES[URL_HISTORY], ".ntrc_2/history1",0);
+	load_strlist(ALL_HISTORIES[PATH_HISTORY],".ntrc_2/history2",1);
+	load_strlist(ALL_HISTORIES[LOG_HISTORY],".ntrc_2/history3",0);
+	load_strlist(ALL_HISTORIES[LOAD_SAVE_HISTORY],".ntrc_2/history4",1);
+	load_strlist(ALL_HISTORIES[USER_HISTORY],".ntrc_2/history5",0);
+	load_strlist(ALL_HISTORIES[PROXY_HISTORY],".ntrc_2/history6",0);
+	load_strlist(ALL_HISTORIES[FILE_HISTORY],".ntrc_2/history7",0);
+	load_strlist(ALL_HISTORIES[USER_AGENT_HISTORY],".ntrc_2/history8",0);
+	load_strlist(ALL_HISTORIES[EXEC_HISTORY],".ntrc_2/history9",0);
+	load_strlist(ALL_HISTORIES[SKIP_HISTORY],".ntrc_2/history11",0);
+	load_strlist(ALL_HISTORIES[SAVE_HISTORY],".ntrc_2/history12",1);
+	load_strlist(ALL_HISTORIES[LOG_SAVE_HISTORY],".ntrc_2/history13",1);
+	load_strlist(ALL_HISTORIES[DESC_HISTORY],".ntrc_2/history14",0);
+	load_strlist(ALL_HISTORIES[REFERER_HISTORY],".ntrc_2/history15",0);
+	load_strlist(ALL_HISTORIES[COOKIE_HISTORY],".ntrc_2/history16",0);
+	load_strlist(ALL_HISTORIES[SOUNDS_HISTORY],".ntrc_2/history17",1);
+	load_strlist(ALL_HISTORIES[THEMES_HISTORY],".ntrc_2/history18",1);
 	if (CFG.REMEMBER_PASS)
-		load_strlist(ALL_HISTORIES[PASS_HISTORY],".ntrc/history10",0);
+		load_strlist(ALL_HISTORIES[PASS_HISTORY],".ntrc_2/history10",0);
 	ALL_HISTORIES[USER_AGENT_HISTORY]->add("%version");
 	ALL_HISTORIES[USER_AGENT_HISTORY]->add("Mozilla/4.05");
 	ALL_HISTORIES[USER_AGENT_HISTORY]->add("Mozilla/4.0 (compatible; MSIE 4.01; Windows 95)");
@@ -378,6 +348,23 @@ static void save_string_to_config(int fd,char *name,char *str) {
 	f_wstr(fd,data);
 };
 
+char *d4x_cfg_search_engines(){
+	int count=D4X_SEARCH_ENGINES.count();
+	d4xSearchEngine *first=D4X_SEARCH_ENGINES.first();
+	char *rval=new char[count+1];
+	char *cur=rval;
+	while(first){
+		if (first->used)
+			*cur='1';
+		else
+			*cur='0';
+		cur++;
+		first=D4X_SEARCH_ENGINES.prev();
+	};
+	*cur=0;
+	return rval;
+};
+
 void save_config() {
 	if (!HOME_VARIABLE)	return;
 	char *cfgpath=compose_path(HOME_VARIABLE,CFG_FILE);
@@ -388,19 +375,12 @@ void save_config() {
 		fd=open(cfgpath,O_TRUNC | O_CREAT |O_RDWR,S_IRUSR | S_IWUSR);
 		delete[] cfg_dir;
 	};
+	if (CFG.SEARCH_ENGINES) delete[] CFG.SEARCH_ENGINES;
+	CFG.SEARCH_ENGINES=d4x_cfg_search_engines();
 	char data[MAX_LEN];
 	if (fd>=0) {
-		list_of_downloads_get_height();
-		list_of_downloads_get_sizes();
+		lod_get_height();
 		fs_list_get_size();
-		CFG.DEFAULT_HOST_LIMIT=LimitsForHosts==NULL?0:LimitsForHosts->get_default_limit();
-		if (FaceForLimits) {
-			FaceForLimits->get_sizes();
-		};
-		for(int i=0;i<=NOTHING_COL;i++){
-			TEMP_COLUMN_POS[i]=ListColumns[i].enum_index;
-			TEMP_COLUMN_SIZES[i]=ListColumns[ListColumns[i].enum_index].size;
-		};
 		int cv_list_len=sizeof(config_variables)/sizeof(struct tConfigVariable);
 		for (int i=0;i<cv_list_len;i++){
 			switch(config_variables[i].type){
@@ -442,25 +422,26 @@ void save_config() {
 			sprintf(data,"Can't write cfgfile to:%s",cfgpath);
 			MainLog->add(data,LOG_ERROR);
 		} else
-			printf("Can't write cfgfile to:%s\n",cfgpath);
+			g_print("Can't write cfgfile to:%s\n",cfgpath);
 	};
-	save_strlist(ALL_HISTORIES[URL_HISTORY], ".ntrc/history1");
-	save_strlist(ALL_HISTORIES[PATH_HISTORY],".ntrc/history2");
-	save_strlist(ALL_HISTORIES[LOG_HISTORY],".ntrc/history3");
-	save_strlist(ALL_HISTORIES[LOAD_SAVE_HISTORY],".ntrc/history4");
-	save_strlist(ALL_HISTORIES[USER_HISTORY],".ntrc/history5");
-	save_strlist(ALL_HISTORIES[PROXY_HISTORY],".ntrc/history6");
-	save_strlist(ALL_HISTORIES[FILE_HISTORY],".ntrc/history7");
-	save_strlist(ALL_HISTORIES[USER_AGENT_HISTORY],".ntrc/history8");
-	save_strlist(ALL_HISTORIES[EXEC_HISTORY],".ntrc/history9");
-	save_strlist(ALL_HISTORIES[PASS_HISTORY],".ntrc/history10");
-	save_strlist(ALL_HISTORIES[SKIP_HISTORY],".ntrc/history11");
-	save_strlist(ALL_HISTORIES[SAVE_HISTORY],".ntrc/history12");
-	save_strlist(ALL_HISTORIES[LOG_SAVE_HISTORY],".ntrc/history13");
-	save_strlist(ALL_HISTORIES[DESC_HISTORY],".ntrc/history14");
-	save_strlist(ALL_HISTORIES[REFERER_HISTORY],".ntrc/history15");
-	save_strlist(ALL_HISTORIES[COOKIE_HISTORY],".ntrc/history16");
-	save_strlist(ALL_HISTORIES[SOUNDS_HISTORY],".ntrc/history17");
+	save_strlist(ALL_HISTORIES[URL_HISTORY], ".ntrc_2/history1");
+	save_strlist(ALL_HISTORIES[PATH_HISTORY],".ntrc_2/history2");
+	save_strlist(ALL_HISTORIES[LOG_HISTORY],".ntrc_2/history3");
+	save_strlist(ALL_HISTORIES[LOAD_SAVE_HISTORY],".ntrc_2/history4");
+	save_strlist(ALL_HISTORIES[USER_HISTORY],".ntrc_2/history5");
+	save_strlist(ALL_HISTORIES[PROXY_HISTORY],".ntrc_2/history6");
+	save_strlist(ALL_HISTORIES[FILE_HISTORY],".ntrc_2/history7");
+	save_strlist(ALL_HISTORIES[USER_AGENT_HISTORY],".ntrc_2/history8");
+	save_strlist(ALL_HISTORIES[EXEC_HISTORY],".ntrc_2/history9");
+	save_strlist(ALL_HISTORIES[PASS_HISTORY],".ntrc_2/history10");
+	save_strlist(ALL_HISTORIES[SKIP_HISTORY],".ntrc_2/history11");
+	save_strlist(ALL_HISTORIES[SAVE_HISTORY],".ntrc_2/history12");
+	save_strlist(ALL_HISTORIES[LOG_SAVE_HISTORY],".ntrc_2/history13");
+	save_strlist(ALL_HISTORIES[DESC_HISTORY],".ntrc_2/history14");
+	save_strlist(ALL_HISTORIES[REFERER_HISTORY],".ntrc_2/history15");
+	save_strlist(ALL_HISTORIES[COOKIE_HISTORY],".ntrc_2/history16");
+	save_strlist(ALL_HISTORIES[SOUNDS_HISTORY],".ntrc_2/history17");
+	save_strlist(ALL_HISTORIES[THEMES_HISTORY],".ntrc_2/history18");
 	delete[] cfgpath;
 };
 
@@ -509,53 +490,6 @@ void load_strlist(tStringList *where,char *what,int normalize) {
 	delete[] path;
 };
 
-void read_limits() {
-	if (!HOME_VARIABLE) return;
-	char *path=compose_path(HOME_VARIABLE,".ntrc/limits");
-	int fd=open(path,O_RDONLY);
-	if (fd>=0) {
-		char temp[MAX_LEN];
-		while (f_rstr(fd,temp,MAX_LEN)) {
-			char temp1[MAX_LEN];
-			char temp2[MAX_LEN];
-			if (f_rstr(fd,temp1,MAX_LEN) && f_rstr(fd,temp2,MAX_LEN)) {
-				int port,upper;
-				sscanf(temp1,"%i",&port);
-				sscanf(temp2,"%i",&upper);
-				LimitsForHosts->add(temp,port,0,upper);
-			} else {
-				printf("bad read\n");
-			};
-		};
-		close(fd);
-	} else {
-		printf(_("Can't open limits configuration file!\n"));
-	};
-};
-
-void save_limits() {
-	if (!HOME_VARIABLE) return;
-	char *path=compose_path(HOME_VARIABLE,".ntrc/limits");
-	int fd=open(path,O_TRUNC | O_CREAT |O_RDWR,S_IRUSR | S_IWUSR);
-	if (fd>=0) {
-		tSortString *tmp=LimitsForHosts->first();
-		while (tmp) {
-			if (tmp->flag==0){
-				char data[MAX_LEN];
-				f_wstr_lf(fd,tmp->body);
-				sprintf(data,"%i",tmp->key);
-				f_wstr_lf(fd,data);
-				sprintf(data,"%i",tmp->upper);
-				f_wstr_lf(fd,data);
-			};
-			tmp=LimitsForHosts->prev();
-		};
-		close(fd);
-	} else {
-		printf(_("Can't write limits to file!!!\n"));
-	};
-};
-
 int downloader_args_type(char *str){
 	DBC_RETVAL_IF_FAIL(str!=NULL,OPT_UNKNOWN);
 	for (int i=0;i<downloader_parsed_args_num;i++){
@@ -582,8 +516,8 @@ int parse_command_line_preload(int argc,char **argv){
 		};
 		case OPT_UNKNOWN:{
 			if (argv[i] && *argv[i]=='-') {
-				printf(_("unknown option:"));
-				printf(" %s\n",argv[i]);
+				g_print(_("unknown option:"));
+				g_print(" %s\n",argv[i]);
 			}
 			break;
 		};
@@ -613,8 +547,93 @@ static void _remote_set_directory_(tMsgClient *clt,char *param){
 	};
 };
 
+static void _do_ls_(char *url,tMsgClient *clt){
+	tPacketStatus status;
+	clt->send_command_short(PACKET_LS,url,strlen(url));
+	while(clt->get_answer_status(&status)){
+		switch(status.Status){
+		case DL_RUN:
+			printf(">");
+			break;
+		case DL_STOP:
+			printf("X");
+			break;
+		case DL_WAIT:
+			printf("o");
+			break;
+		case DL_PAUSE:
+			printf("=");
+			break;
+		case DL_COMPLETE:
+			printf("+");
+			break;
+		case DL_STOPWAIT:
+			printf("-");
+			break;
+		default:
+			printf("?");
+			break;
+		};
+		if (status.url){
+			printf("%s",status.url);
+			delete[] status.url;
+			status.url=NULL;
+		};
+		printf(" %i/%i bytes %i B/s %i/%i attempts\n",
+		       status.Download,status.Size,
+		       status.Speed,
+		       status.Attempt,status.MaxAttempt);
+	};
+};
+
+static void _do_lstree_(tMsgClient *clt){
+	clt->send_command_short(PACKET_LSTREE,NULL,0);
+	char b;
+	int depth=-1;
+	g_print(_("List of queues:\n"));
+	g_print("-----------------------------\n");
+	int N=1;
+	while(clt->readdata(&b,sizeof(b))==sizeof(b)){
+		int len=0;
+		int def_queue=0;
+		switch(b){
+		case LST_SUBQUEUE:
+			depth+=1;
+			break;
+		case LST_DQUEUE:
+			def_queue=1;
+		case LST_QUEUE:{
+			clt->readdata(&len,sizeof(len));
+			printf("[%i] ",N);
+			for (int i=0;i<depth;i++) printf("  ");
+			for (;len>0;len--){
+				clt->readdata(&b,sizeof(b));
+				printf("%c",b);
+			};
+			int num=0,run=0,maxrun=0,complete=0;
+			clt->readdata(&num,sizeof(num));
+			clt->readdata(&run,sizeof(run));
+			clt->readdata(&complete,sizeof(complete));
+			clt->readdata(&maxrun,sizeof(maxrun));
+			printf("\t %i %i %i/%i",num,complete,run,maxrun);
+			if (def_queue) printf("\t<--");
+			printf("\n");
+			N++;
+			break;
+		};
+		case LST_UPQUEUE:{
+			depth-=1;
+			break;
+		};
+		};
+	};
+};
+
+
+
 int parse_command_line_already_run(int argv,char **argc){
 	int rvalue=1;
+	char *referer=0;
 	if (argv>1){
 		tMsgClient *clt=new tMsgClient;
 		for (int i=1;i<argv;i++){
@@ -622,33 +641,64 @@ int parse_command_line_already_run(int argv,char **argc){
 			int option=OPT_UNKNOWN;
 			if (*(argc[i])!='-'){
 				rvalue=0;
-				if (clt->send_command(PACKET_ADD,argc[i],strlen(argc[i])+1)) break;
+				int len=0;
+				char *tmpbuf=compose_strings_array(&len,argc[i],strlen(argc[i])+1,referer);
+				if (clt->send_command(PACKET_ADD,tmpbuf,len)) break;
+				delete [] tmpbuf;
 			}else{
 				option=downloader_args_type(argc[i]);
 				switch(option){
+				case OPT_ADD_OPENLIST:{
+					char *ar=0;
+					int len;
+					while(argv>i+1){
+						i++;
+						if (argc[i][0]==0 || argc[i][0]=='-'){
+							i--;
+							break;
+						};
+						if (ar){
+							char *t=compose_strings_array(&len,ar,len,argc[i]);
+							delete [] ar;
+							
+							ar=t;
+						}else{
+							ar=compose_strings_array(&len,referer?referer:"",
+										      referer?strlen(referer)+1:1,argc[i]);
+						};
+					};
+					if (ar){
+						clt->send_command(PACKET_OPENLIST,ar,len);
+						delete[] ar;
+						rvalue=0;
+					}else{
+						opt_error=1;
+					};
+					break;
+				};
 				case OPT_INFO:{
 					rvalue=0;
 					clt->send_command(PACKET_ASK_RUN,NULL,0);
-					printf(_("Run downloads: %d\n"),clt->get_answer_int());
+					g_print(_("Run downloads: %d\n"),clt->get_answer_int());
 					clt->send_command(PACKET_ASK_PAUSE,NULL,0);
-					printf(_("Paused downloads: %d\n"),clt->get_answer_int());
+					g_print(_("Paused downloads: %d\n"),clt->get_answer_int());
 					clt->send_command(PACKET_ASK_STOP,NULL,0);
-					printf(_("Failed downloads: %d\n"),clt->get_answer_int());
+					g_print(_("Failed downloads: %d\n"),clt->get_answer_int());
 					clt->send_command(PACKET_ASK_COMPLETE,NULL,0);
-					printf(_("Completed downloads: %d\n"),clt->get_answer_int());
-					printf("-------------------------------\n");
+					g_print(_("Completed downloads: %d\n"),clt->get_answer_int());
+					g_print("-------------------------------\n");
 					clt->send_command(PACKET_ASK_FULLAMOUNT,NULL,0);
-					printf(_("Total: %d\n"),clt->get_answer_int());
+					g_print(_("Total: %d\n"),clt->get_answer_int());
 					clt->send_command(PACKET_ASK_READED_BYTES,NULL,0);
-					printf(_("Total bytes loaded: %d\n"),clt->get_answer_int());
+					g_print(_("Total bytes loaded: %d\n"),clt->get_answer_int());
 					clt->send_command(PACKET_ASK_SPEED,NULL,0);
-					printf(_("Current speed: %d\n"),clt->get_answer_int());
+					g_print(_("Current speed: %d\n"),clt->get_answer_int());
 					break;
 				};
 				case OPT_SPEED:{
 					clt->send_command(PACKET_ASK_SPEED,NULL,0);
 					rvalue=0;
-					printf(_("Curent speed: %d\n"),clt->get_answer_int());
+					g_print(_("Curent speed: %d\n"),clt->get_answer_int());
 					break;
 				};
 				case OPT_TRAFFIC_LOW:{
@@ -666,6 +716,15 @@ int parse_command_line_already_run(int argv,char **argc){
 					rvalue=0;
 					break;
 				};
+				case OPT_REFERER:{
+					rvalue=0;
+					if (argv>i+1){
+						i+=1;
+						referer=argc[i];
+					}else
+						opt_error=1;
+					break;
+				};
 				case OPT_SET_DIRECTORY:{
 					rvalue=0;
 					if (argv>i+1){
@@ -723,68 +782,90 @@ int parse_command_line_already_run(int argv,char **argc){
 					break;
 				};
 				case OPT_LS:{
-					tPacketStatus status;
 					rvalue=0;
 					char *which=NULL;
 					if (argv>i+1){
 						i+=1;
 						which=argc[i];
-					}else
-						which="";
-					clt->send_command_short(PACKET_LS,which,strlen(which));
-					while(clt->get_answer_status(&status)){
-						switch(status.Status){
-						case DL_RUN:
-							printf(">");
-							break;
-						case DL_STOP:
-							printf("X");
-							break;
-						case DL_WAIT:
-							printf("o");
-							break;
-						case DL_PAUSE:
-							printf("=");
-							break;
-						case DL_COMPLETE:
-							printf("+");
-							break;
-						case DL_STOPWAIT:
-							printf("-");
-							break;
-						default:
-							printf("?");
-							break;
-						};
-						if (status.url){
-							printf("%s",status.url);
-							delete[] status.url;
-							status.url=NULL;
-						};
-						printf(" %i/%i bytes %i B/s %i/%i attempts\n",
-						       status.Download,status.Size,
-						       status.Speed,
-						       status.Attempt,status.MaxAttempt);
+					};
+					if (which)
+						_do_ls_(which,clt);
+					else
+						_do_lstree_(clt);
+					break;
+				};
+				case OPT_SWITCH:{
+					rvalue=0;
+					if (argv>i+1){
+						i+=1;
+						clt->send_command(PACKET_SWITCH_QUEUE,argc[i],strlen(argc[i]));
+					}else{
+						opt_error=1;
+					};
+					break;
+				};
+				case OPT_ADD_OPEN:{
+					rvalue=0;
+					if (argv>i+1){
+						i+=1;
+						int len=0;
+						char *tmpbuf=compose_strings_array(&len,argc[i],strlen(argc[i])+1,referer);
+						if (clt->send_command(PACKET_ADD_OPEN,tmpbuf,len)) break;
+						delete [] tmpbuf;
+					}else{
+						opt_error=1;
 					};
 					break;
 				};
 				};
 			};
 			if (opt_error && downloader_args_errors[option])
-				printf("%s\n",_(downloader_args_errors[option]));
+				g_print("%s\n",_(downloader_args_errors[option]));
 		};		
 		delete clt;
 	};
 	return rvalue;
 };
 
+void init_add_dnd_window(const char *url,const char *desc,const char *ref=0);
+void create_addlinks_with_referer(const std::vector<std::string> &v,const char *savepath);
+
 void parse_command_line_postload(int argv,char **argc){
+	char *referer=0;
 	for (int i=1;i<argv;i++){
 		if (*(argc[i])!='-'){
-			aa.add_downloading(argc[i]);
+			_aa_.add_downloading(argc[i],0,0,0,referer);
 		};
 		int option=downloader_args_type(argc[i]);
 		switch (option){
+		case OPT_REFERER:{
+			if (argv>i+1){
+				i+=1;
+				referer=argc[i];
+			};
+			break;
+		};
+		case OPT_ADD_OPEN:{
+			if (argv>i+1){
+				i++;
+				init_add_dnd_window(argc[i],_("Added from command line"),referer);
+			};
+			break;
+		};
+		case OPT_ADD_OPENLIST:{
+			std::vector<std::string> v;
+			v.push_back(referer?referer:"");
+			while(argv>i+1){
+				i++;
+				if (argc[i][0]==0 || argc[i][0]=='-'){
+					i--;
+					break;
+				};
+				v.push_back(argc[i]);
+			};
+			create_addlinks_with_referer(v,CFG.GLOBAL_SAVE_PATH);
+			break;
+		};
 		case OPT_TRAFFIC_LOW:{
 			CFG.SPEED_LIMIT=1;
 			set_speed_buttons();
@@ -800,17 +881,10 @@ void parse_command_line_postload(int argv,char **argc){
 			set_speed_buttons();
 			break;
 		};
-		case OPT_SET_MAX_THREADS:{
-			if (argv<=i+1 || sscanf(argc[i+1],"%d",&CFG.MAX_THREADS)<1){
-				printf("%s\n",_(downloader_args_errors[OPT_SET_MAX_THREADS]));
-				i+=1;
-			};
-			break;
-		};
 		case OPT_EXIT_TIME:
 			int tmp;
 			if (argv<=i+1 || sscanf(argc[i+1],"%d",&tmp)<1 || tmp<0)
-				printf("%s\n",_(downloader_args_errors[OPT_EXIT_TIME]));
+				g_print("%s\n",_(downloader_args_errors[OPT_EXIT_TIME]));
 			else{
 				if (tmp>=1){
 					CFG.EXIT_COMPLETE=1;
@@ -829,7 +903,7 @@ void parse_command_line_postload(int argv,char **argc){
 				i+=1;
 				CFG.GLOBAL_SAVE_PATH=copy_string(argc[i]);
 			}else
-				printf("%s\n",_(downloader_args_errors[OPT_SET_DIRECTORY]));
+				g_print("%s\n",_(downloader_args_errors[OPT_SET_DIRECTORY]));
 			break;
 		};
 		case OPT_GEOMETRY:{
@@ -849,7 +923,7 @@ void parse_command_line_postload(int argv,char **argc){
 //					printf("x=%i y=%i\n",x,y);
 				};
 			}else
-				printf("%s\n",_(downloader_args_errors[OPT_GEOMETRY]));
+				g_print("%s\n",_(downloader_args_errors[OPT_GEOMETRY]));
 		};
 		};
 	};
@@ -869,25 +943,28 @@ void help_print_args(int type){
 };
 
 void help_print(){
-	printf(_("Usage: nt [OPTION] ... [URL]"));printf("\n\n");
-	help_print_args(OPT_HELP);printf(_("print this page and exit"));printf("\n");
-	help_print_args(OPT_VERSION);printf(_("show version information and exit"));printf("\n");
-	help_print_args(OPT_INFO);printf(_("show information if already run"));printf("\n");
-	help_print_args(OPT_SPEED);printf(_("show current speed if already run"));printf("\n");
-	help_print_args(OPT_RUN_MINIMIZED);printf(_("run in minimized mode"));printf("\n");
-	help_print_args(OPT_EXIT_TIME);printf(_("set timeout for exiting if nothing to do"));printf("\n");
-	help_print_args(OPT_SET_DIRECTORY);printf(_("set directory for saving files"));printf("\n");
-	help_print_args(OPT_TRAFFIC_LOW);printf(_("set lower speed limitation"));printf("\n");
-	help_print_args(OPT_TRAFFIC_MIDDLE);printf(_("set medium speed limitation"));printf("\n");
-	help_print_args(OPT_TRAFFIC_HIGH);printf(_("set unlimited speed"));printf("\n");
-	help_print_args(OPT_DEL_COMPLETED);printf(_("delete completed if already run"));printf("\n");
-	help_print_args(OPT_SET_MAX_THREADS);printf(_("set maximum active downloads"));printf("\n");
-	help_print_args(OPT_RERUN_FAILED);printf(_("restart all failed downloads"));printf("\n");
-	help_print_args(OPT_WITHOUT_FACE);printf(_("run program without X interface"));printf("\n");
-	help_print_args(OPT_LS);printf(_("display info about URL in queue of downloads"));printf("\n");
-	help_print_args(OPT_DEL);printf(_("remove a download from queue"));printf("\n");
-	help_print_args(OPT_STOP);printf(_("stop a download"));printf("\n");
-	help_print_args(OPT_COLOR);printf(_("using colors if run without interface"));printf("\n");
-	help_print_args(OPT_GEOMETRY);printf(_("specifies preferred size and position of main window"));printf("\n");
+	g_print(_("Usage: nt [OPTION] ... [URL]"));printf("\n\n");
+	help_print_args(OPT_HELP);g_print(_("print this page and exit"));printf("\n");
+	help_print_args(OPT_VERSION);g_print(_("show version information and exit"));printf("\n");
+	help_print_args(OPT_INFO);g_print(_("show information if already run"));printf("\n");
+	help_print_args(OPT_SPEED);g_print(_("show current speed if already run"));printf("\n");
+	help_print_args(OPT_RUN_MINIMIZED);g_print(_("run in minimized mode"));printf("\n");
+	help_print_args(OPT_EXIT_TIME);g_print(_("set timeout for exiting if nothing to do"));printf("\n");
+	help_print_args(OPT_SET_DIRECTORY);g_print(_("set directory for saving files"));printf("\n");
+	help_print_args(OPT_TRAFFIC_LOW);g_print(_("set lower speed limitation"));printf("\n");
+	help_print_args(OPT_TRAFFIC_MIDDLE);g_print(_("set medium speed limitation"));printf("\n");
+	help_print_args(OPT_TRAFFIC_HIGH);g_print(_("set unlimited speed"));printf("\n");
+	help_print_args(OPT_DEL_COMPLETED);g_print(_("delete completed if already run"));printf("\n");
+	help_print_args(OPT_SET_MAX_THREADS);g_print(_("set maximum active downloads"));printf("\n");
+	help_print_args(OPT_RERUN_FAILED);g_print(_("restart all failed downloads"));printf("\n");
+	help_print_args(OPT_WITHOUT_FACE);g_print(_("run program without X interface"));printf("\n");
+	help_print_args(OPT_LS);g_print(_("display info about URL in queue of downloads"));printf("\n");
+	help_print_args(OPT_SWITCH);g_print(_("change current default queue"));printf("\n");
+	help_print_args(OPT_DEL);g_print(_("remove a download from queue"));printf("\n");
+	help_print_args(OPT_STOP);g_print(_("stop a download"));printf("\n");
+	help_print_args(OPT_COLOR);g_print(_("using colors if run without interface"));printf("\n");
+	help_print_args(OPT_GEOMETRY);g_print(_("specifies preferred size and position of main window"));printf("\n");
+	help_print_args(OPT_REFERER);g_print(_("specify rerferer for next added URL"));printf("\n");
+	help_print_args(OPT_ADD_OPEN);g_print(_("open \"Add new download\" dialog for the URL"));printf("\n");
 	printf("\n");
 };
