@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -15,7 +15,7 @@ extern GtkWidget *AddWindow;
 extern tDList *list_for_adding;
 void init_add_window(...);
 void init_add_clipboard_window(...);
-void init_add_dnd_window(char *url,char *desc);
+void init_add_dnd_window(const char *url,const char *desc,const char *ref=0);
 void init_edit_common_properties_window(int *array);
 void d4x_automated_add();
 
