@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -8,6 +8,7 @@
  *	but WITHOUT ANY WARRANTY; without even the implied warranty of
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
+
 #include "colors.h"
 const GdkColor LBLUE = {
     0, 0xbb00, 0xdd00, 0xffff
@@ -22,7 +23,7 @@ const GdkColor RED = {
     0, 0x8000, 0x0000, 0x0000
 };
 const GdkColor LRED = {
-    0, 0xffff, 0xdd00, 0xdd00
+    0, 0xffff, 0x5500, 0x5500
 };
 const GdkColor LCYAN = {
     0, 0xdd00, 0xeeff, 0xffff
@@ -54,6 +55,9 @@ const GdkColor GREY = {
 const GdkColor LIGHTGREY = {
     0, 0xee00, 0xee00, 0xee00
 };
+const GdkColor MIDGREY = {
+    0, 0xaa00, 0xaa00, 0xaa00
+};
 const GdkColor DARKGREY = {
-    0, 0x6600, 0x6600, 0xd660
+    0, 0x6600, 0x6600, 0x6600
 };
