@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -14,8 +14,8 @@
 #include <gtk/gtk.h>
 #include "../filter.h"
 
-void d4x_filters_window_add(d4xFNode *filter);
-void d4x_filters_window_init();
+void d4x_filters_window_add(const d4x::Filter &filter);
+GtkWidget *d4x_filters_window_init();
 void d4x_filters_window_destroy();
 
 
