@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -8,6 +8,7 @@
  *	but WITHOUT ANY WARRANTY; without even the implied warranty of
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
+
 #include "fsched.h"
 #include "../schedule.h"
 #include "../var.h"
@@ -15,9 +16,11 @@
 #include "misc.h"
 #include "mywidget.h"
 #include "edit.h"
+#include "list.h"
 
 GtkWidget *d4x_scheduler_window=(GtkWidget *)NULL;
-GtkWidget *d4x_scheduler_clist=(GtkWidget *)NULL;
+GtkTreeView *d4x_scheduler_view=(GtkTreeView *)NULL;
+GtkListStore *d4x_scheduler_store=(GtkListStore *)NULL;
 
 /*
 static void d4x_scheduler_select(GtkWidget *clist, gint row, gint column,
@@ -43,7 +46,7 @@ static char *action_names[]={
 
 void d4x_scheduler_insert(d4xSchedAction *act,d4xSchedAction *prev){
 	if (d4x_scheduler_window==NULL) return;
-	char *text[2];
+	char *text[3];
 	char buf[MAX_LEN];
 	char buf2[MAX_LEN];
 	struct tm tm_time;
@@ -71,10 +74,8 @@ void d4x_scheduler_insert(d4xSchedAction *act,d4xSchedAction *prev){
 	};
 	case SACT_ADD_DOWNLOAD:{
 		d4xSAAddDownload *a=(d4xSAAddDownload *)act;
-		if (a->dwn && a->dwn->info){
-			char *url=a->dwn->info->url();
-			g_snprintf(buf2,MAX_LEN,"%s",url);
-			delete[] url;
+		if (a->dwn){
+			g_snprintf(buf2,MAX_LEN,"%s",std::string(a->dwn->info).c_str());
 		};
 		break;
 	};
@@ -83,25 +84,30 @@ void d4x_scheduler_insert(d4xSchedAction *act,d4xSchedAction *prev){
 	case SACT_RUN_DOWNLOAD:
 	case SACT_PAUSE_DOWNLOAD:{
 		d4xSAUrl *a=(d4xSAUrl *)act;
-		if (a->url){
-			char *url=a->url->url();
-			g_snprintf(buf2,MAX_LEN,"%s",url);
-			delete[] url;
+		if (a->url.is_valid()){
+			g_snprintf(buf2,MAX_LEN,"%s",std::string(a->url).c_str());
 		};
 		break;
 	};
 	};
-	gint row=0;
+	GtkTreeIter iter;
 	if (prev)
-		row=gtk_clist_find_row_from_data(GTK_CLIST(d4x_scheduler_clist),prev)+1;
-	row=gtk_clist_insert(GTK_CLIST(d4x_scheduler_clist),row,text);
-	gtk_clist_set_row_data(GTK_CLIST(d4x_scheduler_clist),row,act);
+		gtk_list_store_insert_after(d4x_scheduler_store,&iter,prev->iter);
+	else
+		gtk_list_store_insert_after(d4x_scheduler_store,&iter,NULL);
+	gtk_list_store_set(d4x_scheduler_store, &iter,
+			   0,text[0],
+			   1,text[1],
+			   2,text[2],
+			   3,act,
+			   -1);
+	if (act->iter) gtk_tree_iter_free(act->iter);
+	act->iter=gtk_tree_iter_copy(&iter);
 };
 
 void d4x_scheduler_remove(d4xSchedAction *act){
 	if (d4x_scheduler_window==NULL) return;
-	gint row=gtk_clist_find_row_from_data(GTK_CLIST(d4x_scheduler_clist),act);
-	gtk_clist_remove(GTK_CLIST(d4x_scheduler_clist),row);
+	gtk_list_store_remove(d4x_scheduler_store,act->iter);
 };
 
 static gint compare_nodes2(gconstpointer a,gconstpointer b){
@@ -114,18 +120,20 @@ static gint compare_nodes2(gconstpointer a,gconstpointer b){
 
 void d4x_scheduler_remove_selected(){
 	if (d4x_scheduler_window==NULL) return;
-	GList *select=g_list_copy(GTK_CLIST(d4x_scheduler_clist)->selection);
-	select=g_list_sort(select,compare_nodes2);
-	while (select) {
-		gint row=GPOINTER_TO_INT(select->data);
-		d4xSchedAction *act=(d4xSchedAction *)gtk_clist_get_row_data(GTK_CLIST(d4x_scheduler_clist),row);
-		if (act){
-			MainScheduler->del_action(act);
-			delete(act);
-		};
-		select=select->next;
+	GtkTreeSelection *s=gtk_tree_view_get_selection(d4x_scheduler_view);
+	tQueue q;
+	gtk_tree_selection_selected_foreach(s,
+					    _foreach_remove_prepare_,
+					    &q);
+	tNode *t=q.last();
+	while(t){
+		GValue val={0,};
+		gtk_tree_model_get_value(GTK_TREE_MODEL(d4x_scheduler_store),((tmpIterNode*)t)->iter,3,&val);
+		d4xSchedAction *act=(d4xSchedAction *)g_value_get_pointer(&val);
+		g_value_unset(&val);
+		MainScheduler->del_action(act);
+		t=t->next;
 	};
-	g_list_free(select);
 };
 
 gint d4x_scheduler_close() {
@@ -141,24 +149,38 @@ void d4x_scheduler_init_editor(){
 
 	gtk_widget_show_all(tmp);
 	gtk_window_set_modal (GTK_WINDOW(tmp),TRUE);
-	gtk_window_set_transient_for (GTK_WINDOW (tmp), GTK_WINDOW (d4x_scheduler_window));
+	gtk_window_set_transient_for (GTK_WINDOW (tmp), GTK_WINDOW (MainWindow));
+};
+
+static GtkTreeIter *_sel_iter_=(GtkTreeIter *)NULL;
+
+static void _foreach_edit_prepare_(GtkTreeModel *model,GtkTreePath *path,
+				   GtkTreeIter *iter,gpointer p){
+	if (_sel_iter_) gtk_tree_iter_free(_sel_iter_);
+	_sel_iter_=gtk_tree_iter_copy(iter);
 };
 
 void d4x_scheduler_edit(){
 	if (d4x_scheduler_window==NULL) return;
-	GList *select=GTK_CLIST(d4x_scheduler_clist)->selection;
-	if (select){		
-		d4xSchedAction *act=(d4xSchedAction *)gtk_clist_get_row_data(GTK_CLIST(d4x_scheduler_clist),
-									     GPOINTER_TO_INT(select->data));
+	GtkTreeSelection *s=gtk_tree_view_get_selection(d4x_scheduler_view);
+	gtk_tree_selection_selected_foreach(s,
+					    _foreach_edit_prepare_,
+					    NULL);
+	if (_sel_iter_){
+		GValue val={0,};
+		gtk_tree_model_get_value(GTK_TREE_MODEL(d4x_scheduler_store),_sel_iter_,3,&val);
+		d4xSchedAction *act=(d4xSchedAction *)g_value_get_pointer(&val);
+		g_value_unset(&val);
 		if (act && act->lock==0){
 			act->lock=1;
 			GtkWidget *tmp=my_gtk_aeditor_new(act);
-//			MyGtkAEditor *editor=MY_GTK_AEDITOR(tmp);
 			gtk_widget_show_all(tmp);
 			gtk_window_set_modal (GTK_WINDOW(tmp),TRUE);
-			gtk_window_set_transient_for (GTK_WINDOW (tmp), GTK_WINDOW (d4x_scheduler_window));
+			gtk_window_set_transient_for (GTK_WINDOW (tmp), GTK_WINDOW (MainWindow));
 		};
+		gtk_tree_iter_free(_sel_iter_);
 	};
+	_sel_iter_=(GtkTreeIter *)NULL;
 };
 
 gint d4x_scheduler_select(GtkWidget *widget, gint row, gint column,
@@ -170,80 +192,75 @@ gint d4x_scheduler_select(GtkWidget *widget, gint row, gint column,
 	return(TRUE);
 };
 
-void d4x_scheduler_init(){
+GtkWidget *d4x_scheduler_init(){
 	if (d4x_scheduler_window) {
-		gdk_window_show(d4x_scheduler_window->window);
-		return;
+		return(d4x_scheduler_window);
 	};
-	d4x_scheduler_window = gtk_window_new(GTK_WINDOW_DIALOG);
-	gtk_window_set_title(GTK_WINDOW (d4x_scheduler_window),
-			     _("Scheduler"));
-	gtk_window_set_wmclass(GTK_WINDOW(d4x_scheduler_window),
-			       "D4X_Scheduler","D4X");
-	gtk_window_set_position(GTK_WINDOW(d4x_scheduler_window),GTK_WIN_POS_CENTER);
-	gtk_widget_set_usize(d4x_scheduler_window,-1,400);
-	gtk_container_border_width(GTK_CONTAINER(d4x_scheduler_window),5);
 	gchar *titles[]={_("Time"),_("Action"),_("Info")};
-	d4x_scheduler_clist = gtk_clist_new_with_titles(3, titles);
-	gtk_clist_set_shadow_type (GTK_CLIST(d4x_scheduler_clist), GTK_SHADOW_IN);
-	gtk_clist_set_column_auto_resize(GTK_CLIST(d4x_scheduler_clist),0,TRUE);
-	gtk_clist_set_column_auto_resize(GTK_CLIST(d4x_scheduler_clist),1,TRUE);
-	gtk_clist_set_column_auto_resize(GTK_CLIST(d4x_scheduler_clist),2,TRUE);
-//	gtk_clist_set_column_width (GTK_CLIST(d4x_scheduler_clist), 1 , 200);
-	gtk_clist_set_selection_mode(GTK_CLIST(d4x_scheduler_clist),GTK_SELECTION_EXTENDED);
-	gtk_signal_connect(GTK_OBJECT(d4x_scheduler_clist), "select_row",
-	                   GTK_SIGNAL_FUNC(d4x_scheduler_select),NULL);
+	GtkListStore *list_store = gtk_list_store_new(4,
+						      G_TYPE_STRING,
+						      G_TYPE_STRING,
+						      G_TYPE_STRING,
+						      G_TYPE_POINTER);
+	d4x_scheduler_store=list_store;
+	d4x_scheduler_view = GTK_TREE_VIEW(gtk_tree_view_new_with_model(GTK_TREE_MODEL(list_store)));
+	gtk_tree_view_set_headers_visible(d4x_scheduler_view,TRUE);
+	GtkCellRenderer *renderer;
+	GtkTreeViewColumn *col;
+	for (int i=0;i<3;i++){
+		renderer = gtk_cell_renderer_text_new();
+		col=gtk_tree_view_column_new_with_attributes(_(titles[i]),
+							     renderer,
+							     "text",i,
+							     NULL);
+		gtk_tree_view_column_set_visible(col,TRUE);
+		gtk_tree_view_append_column(d4x_scheduler_view,col);
+	};
+
+//	gtk_clist_set_selection_mode(GTK_CLIST(d4x_scheduler_clist),GTK_SELECTION_EXTENDED);
+//	g_signal_connect(G_OBJECT(d4x_scheduler_clist), "select_row",
+//	                   G_CALLBACK(d4x_scheduler_select),NULL);
 
 	GtkWidget *scroll_window=gtk_scrolled_window_new((GtkAdjustment *)NULL,(GtkAdjustment *)NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll_window),
 	                                GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
-	gtk_container_add(GTK_CONTAINER(scroll_window),d4x_scheduler_clist);
-	GtkWidget *button=gtk_button_new_with_label(_("Ok"));
-	GtkWidget *add_button=gtk_button_new_with_label(_("New"));
-	GtkWidget *edit_button=gtk_button_new_with_label(_("Edit"));
-	GtkWidget *del_button=gtk_button_new_with_label(_("Remove"));
-	GTK_WIDGET_SET_FLAGS(button,GTK_CAN_DEFAULT);
+	gtk_container_add(GTK_CONTAINER(scroll_window),GTK_WIDGET(d4x_scheduler_view));
+	GtkWidget *add_button=gtk_button_new_from_stock(GTK_STOCK_ADD);
+	GtkWidget *edit_button=gtk_button_new_from_stock(GTK_STOCK_PROPERTIES);
+	GtkWidget *del_button=gtk_button_new_from_stock(GTK_STOCK_REMOVE);
 	GTK_WIDGET_SET_FLAGS(edit_button,GTK_CAN_DEFAULT);
 	GTK_WIDGET_SET_FLAGS(add_button,GTK_CAN_DEFAULT);
 	GTK_WIDGET_SET_FLAGS(del_button,GTK_CAN_DEFAULT);
-	GtkWidget *vbox=gtk_vbox_new(FALSE,0);
+	GtkWidget *vbox=d4x_scheduler_window=gtk_vbox_new(FALSE,5);
+	GtkWidget *label=gtk_label_new(_("Scheduler"));
+
 	GtkWidget *hbox=gtk_hbutton_box_new();
-	gtk_box_set_spacing(GTK_BOX(vbox),5);
+	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox),GTK_BUTTONBOX_START);
 	gtk_box_set_spacing(GTK_BOX(hbox),3);
+	gtk_box_pack_start(GTK_BOX(vbox),my_gtk_set_header_style(label),FALSE,FALSE,0);
 	gtk_box_pack_start(GTK_BOX(vbox),scroll_window,TRUE,TRUE,0);
 	gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,0);
 	gtk_box_pack_end(GTK_BOX(hbox),add_button,FALSE,FALSE,0);
 	gtk_box_pack_end(GTK_BOX(hbox),del_button,FALSE,FALSE,0);
 	gtk_box_pack_end(GTK_BOX(hbox),edit_button,FALSE,FALSE,0);
-	gtk_box_pack_end(GTK_BOX(hbox),button,FALSE,FALSE,0);
-	gtk_container_add(GTK_CONTAINER(d4x_scheduler_window),vbox);
-	gtk_window_set_default(GTK_WINDOW(d4x_scheduler_window),button);
 	MainScheduler->redraw();
-	gtk_widget_show_all(d4x_scheduler_window);
 
-	gtk_signal_connect(GTK_OBJECT(d4x_scheduler_window),"delete_event",GTK_SIGNAL_FUNC(d4x_scheduler_close), NULL);
-	gtk_signal_connect(GTK_OBJECT(add_button),"clicked",GTK_SIGNAL_FUNC(d4x_scheduler_init_editor),NULL);
-	gtk_signal_connect(GTK_OBJECT(del_button),"clicked",GTK_SIGNAL_FUNC(d4x_scheduler_remove_selected),NULL);
-	gtk_signal_connect(GTK_OBJECT(edit_button),"clicked",GTK_SIGNAL_FUNC(d4x_scheduler_edit),NULL);
-	gtk_signal_connect(GTK_OBJECT(button),"clicked",GTK_SIGNAL_FUNC(d4x_scheduler_close),NULL);
-/*
-	gtk_signal_connect(GTK_OBJECT(d4x_scheduler_clist),"select_row",GTK_SIGNAL_FUNC(d4x_scheduler_select),NULL);
-	gtk_signal_connect(GTK_OBJECT(d4x_scheduler_clist),"event",GTK_SIGNAL_FUNC(face_limits_list_event_callback),this);
-	gtk_signal_connect(GTK_OBJECT(del_button),"clicked",GTK_SIGNAL_FUNC(face_limits_del),this);
-*/
+	g_signal_connect(G_OBJECT(add_button),"clicked",G_CALLBACK(d4x_scheduler_init_editor),NULL);
+	g_signal_connect(G_OBJECT(del_button),"clicked",G_CALLBACK(d4x_scheduler_remove_selected),NULL);
+	g_signal_connect(G_OBJECT(edit_button),"clicked",G_CALLBACK(d4x_scheduler_edit),NULL);
+	gtk_widget_ref(d4x_scheduler_window);
+	return(d4x_scheduler_window);
 };
 
 /*********************************************************************/
 
-static GtkWidgetClass *parent_class = (GtkWidgetClass *)NULL;
+static GtkWidgetClass *parent_class2 = (GtkWidgetClass *)NULL;
 
 static void my_gtk_aeditor_edit_ok(GtkWidget *widget,MyGtkAEditor *editor){
 	tDownload *what=editor->dwn;
 	what->editor->apply_changes();
 	what->delete_editor();
-	char *url=what->info->url();
-	text_to_combo(editor->url_entry,url);
-	delete[] url;
+	text_to_combo(editor->url_entry,std::string(editor->dwn->info).c_str());
 };
 
 static void my_gtk_aeditor_edit_download(GtkWidget *widget,MyGtkAEditor *editor){
@@ -251,32 +268,26 @@ static void my_gtk_aeditor_edit_download(GtkWidget *widget,MyGtkAEditor *editor)
 	int flag=0;
 	if (editor->dwn==NULL){
 		editor->dwn=new tDownload;
-		editor->dwn->set_default_cfg();
-		editor->dwn->config.save_path.set(CFG.GLOBAL_SAVE_PATH);
 		char *url_entry_cont=text_from_combo(editor->url_entry);
-		editor->dwn->info=new tAddr(url_entry_cont);
-		if (CFG.USE_PROXY_FOR_FTP) {
-			editor->dwn->config.proxy_host.set(CFG.FTP_PROXY_HOST);
-			editor->dwn->config.proxy_port=CFG.FTP_PROXY_PORT;
-			if (CFG.NEED_PASS_FTP_PROXY) {
-				editor->dwn->config.proxy_user.set(CFG.FTP_PROXY_USER);
-				editor->dwn->config.proxy_pass.set(CFG.FTP_PROXY_PASS);
-			};
-		};
-		editor->dwn->config.proxy_type=CFG.FTP_PROXY_TYPE;
+		editor->dwn->info=std::string(url_entry_cont);
 		if (url_entry_cont==NULL || *url_entry_cont==0)
 			flag=1;
 	};
+	if (editor->dwn->config==NULL){
+		editor->dwn->config=new tCfg;
+		editor->dwn->set_default_cfg();
+		editor->dwn->config->save_path.set(CFG.GLOBAL_SAVE_PATH);
+	};
 	what=editor->dwn;
-	init_edit_window_without_ok(what);
+	init_edit_window_without_ok(what,1);
 	what->editor->disable_time();
 	if (flag)
 		what->editor->clear_url();
 	gtk_window_set_modal (GTK_WINDOW(what->editor->window),TRUE);
 	gtk_window_set_transient_for (GTK_WINDOW (what->editor->window), GTK_WINDOW (editor));
-	gtk_signal_connect(GTK_OBJECT(what->editor->ok_button),"clicked",
-			   GTK_SIGNAL_FUNC(my_gtk_aeditor_edit_ok),
-			   editor);
+	g_signal_connect(G_OBJECT(what->editor->ok_button),"clicked",
+			 G_CALLBACK(my_gtk_aeditor_edit_ok),
+			 editor);
 };
 
 static void my_gtk_aeditor_browse_ok(GtkWidget *widget,MyGtkAEditor *editor){
@@ -300,12 +311,13 @@ static void my_gtk_aeditor_browse(GtkWidget *widget,MyGtkAEditor *editor){
 	if (tmp && *tmp)
 		gtk_file_selection_set_filename(GTK_FILE_SELECTION(browser),tmp);
 
-	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(browser)->ok_button),
-			   "clicked",GTK_SIGNAL_FUNC(my_gtk_aeditor_browse_ok),editor);
-	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(browser)->cancel_button),
-			   "clicked",GTK_SIGNAL_FUNC(my_gtk_aeditor_browse_cancel),editor);
-	gtk_signal_connect(GTK_OBJECT(&(GTK_FILE_SELECTION(browser)->window)),
-			   "delete_event",GTK_SIGNAL_FUNC(my_gtk_aeditor_browse_delete),editor);
+	g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(browser)->ok_button),
+			 "clicked",G_CALLBACK(my_gtk_aeditor_browse_ok),editor);
+	g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(browser)->cancel_button),
+			 "clicked",G_CALLBACK(my_gtk_aeditor_browse_cancel),editor);
+	g_signal_connect(G_OBJECT(browser),
+			 "delete_event",G_CALLBACK(my_gtk_aeditor_browse_delete),editor);
+	d4x_eschandler_init(GTK_WIDGET(browser),editor);
 	editor->browser=browser;
 	gtk_widget_show(browser);
 	gtk_window_set_modal (GTK_WINDOW(browser),TRUE);
@@ -340,31 +352,29 @@ static void aeditor_select_mode_int(MyGtkAEditor *editor,int i){
 		};
 		switch(i){
 		case SACT_SET_SPEED:{
-			GtkWidget *hbox=gtk_hbox_new(FALSE,0);
-			gtk_box_set_spacing(GTK_BOX(hbox),5);
+			GtkWidget *hbox=gtk_hbox_new(FALSE,5);
 			editor->frame_child=hbox;
 			
 			editor->sb_low=gtk_radio_button_new_with_label((GSList *)NULL,
 								       _(SPEED_LIMITATIONS_NAMES[1]));
 			gtk_box_pack_start(GTK_BOX(hbox),editor->sb_low,
 					   FALSE,FALSE,0);
-			GSList *group1=gtk_radio_button_group(GTK_RADIO_BUTTON(editor->sb_low));
+			GSList *group1=gtk_radio_button_get_group(GTK_RADIO_BUTTON(editor->sb_low));
 			editor->sb_middle=gtk_radio_button_new_with_label(group1,
 									  _(SPEED_LIMITATIONS_NAMES[2]));
 			gtk_box_pack_start(GTK_BOX(hbox),editor->sb_middle,
 					   FALSE,FALSE,0);
-			editor->sb_high=gtk_radio_button_new_with_label(gtk_radio_button_group(
-				GTK_RADIO_BUTTON(editor->sb_middle)),_(SPEED_LIMITATIONS_NAMES[3]));
+			editor->sb_high=gtk_radio_button_new_with_label(gtk_radio_button_get_group(GTK_RADIO_BUTTON(editor->sb_middle)),_(SPEED_LIMITATIONS_NAMES[3]));
 			gtk_box_pack_start(GTK_BOX(hbox),
 					   editor->sb_high,FALSE,FALSE,0);
 			gtk_container_add(GTK_CONTAINER(editor->frame),hbox);
 			if (editor->action){
 				d4xSASpeed *act=(d4xSASpeed *)(editor->action);
 				if (act->speed==1)
-					gtk_signal_emit_by_name(GTK_OBJECT(editor->sb_low), "clicked");
+					g_signal_emit_by_name(G_OBJECT(editor->sb_low), "clicked");
 				else
-					gtk_signal_emit_by_name(act->speed==2?GTK_OBJECT(editor->sb_middle):GTK_OBJECT(editor->sb_high),
-								"clicked");
+					g_signal_emit_by_name(act->speed==2?G_OBJECT(editor->sb_middle):G_OBJECT(editor->sb_high),
+							      "clicked");
 			};
 			break;
 		};
@@ -373,8 +383,7 @@ static void aeditor_select_mode_int(MyGtkAEditor *editor,int i){
 		case SACT_DEL_IF_COMPLETED:
 		case SACT_DELETE_DOWNLOAD:{
 			editor->url_entry=my_gtk_combo_new(ALL_HISTORIES[URL_HISTORY]);//gtk_entry_new();
-			GtkWidget *hbox=gtk_hbox_new(FALSE,0);
-			gtk_box_set_spacing(GTK_BOX(hbox),5);
+			GtkWidget *hbox=gtk_hbox_new(FALSE,5);
 			editor->frame_child=hbox;
 			gtk_box_pack_start(GTK_BOX(hbox),
 					   gtk_label_new("URL:"),
@@ -384,18 +393,15 @@ static void aeditor_select_mode_int(MyGtkAEditor *editor,int i){
 			gtk_container_add(GTK_CONTAINER(editor->frame),hbox);
 			if (editor->action && _is_sa_url_(editor->action->type())){
 				d4xSAUrl *act=(d4xSAUrl *)(editor->action);
-				if (act->url){
-					char *url=act->url->url();
-					text_to_combo(editor->url_entry,url);
-					delete[] url;
+				if (act->url.is_valid()){
+					text_to_combo(editor->url_entry,std::string(act->url).c_str());
 				};
 			};
 			break;
 		};
 		case SACT_ADD_DOWNLOAD:{
 			editor->url_entry=my_gtk_combo_new(ALL_HISTORIES[URL_HISTORY]);//gtk_entry_new();
-			GtkWidget *hbox=gtk_hbox_new(FALSE,0);
-			gtk_box_set_spacing(GTK_BOX(hbox),5);
+			GtkWidget *hbox=gtk_hbox_new(FALSE,5);
 			editor->frame_child=hbox;
 			gtk_box_pack_start(GTK_BOX(hbox),
 					   gtk_label_new("URL:"),
@@ -408,18 +414,16 @@ static void aeditor_select_mode_int(MyGtkAEditor *editor,int i){
 				editor->dwn->copy(a->dwn);
 			};
 			if (editor->dwn){
-				if (editor->dwn && editor->dwn->info){
-					char *url=editor->dwn->info->url();
-					text_to_combo(editor->url_entry,url);
-					delete[] url;
+				if (editor->dwn && editor->dwn->info.is_valid()){
+					text_to_combo(editor->url_entry,std::string(editor->dwn->info).c_str());
 				}else
-				text_to_combo(editor->url_entry,"");
+					text_to_combo(editor->url_entry,"");
 			}else
 				text_to_combo(editor->url_entry,"");
-			GtkWidget *button=gtk_button_new_with_label(_("Edit"));
-			gtk_signal_connect(GTK_OBJECT(button),"clicked",
-					   GTK_SIGNAL_FUNC(my_gtk_aeditor_edit_download),
-					   editor);
+			GtkWidget *button=gtk_button_new_from_stock(GTK_STOCK_PROPERTIES);
+			g_signal_connect(G_OBJECT(button),"clicked",
+					 G_CALLBACK(my_gtk_aeditor_edit_download),
+					 editor);
 			gtk_box_pack_start(GTK_BOX(hbox),button,
 					   FALSE,FALSE,0);
 			gtk_container_add(GTK_CONTAINER(editor->frame),hbox);
@@ -427,8 +431,7 @@ static void aeditor_select_mode_int(MyGtkAEditor *editor,int i){
 		};
 		case SACT_EXECUTE:{
 			editor->path_entry=gtk_entry_new();
-			GtkWidget *hbox=gtk_hbox_new(FALSE,0);
-			gtk_box_set_spacing(GTK_BOX(hbox),5);
+			GtkWidget *hbox=gtk_hbox_new(FALSE,5);
 			editor->frame_child=hbox;
 			gtk_box_pack_start(GTK_BOX(hbox),
 					   gtk_label_new(_("Command:")),
@@ -436,8 +439,8 @@ static void aeditor_select_mode_int(MyGtkAEditor *editor,int i){
 			gtk_box_pack_start(GTK_BOX(hbox),editor->path_entry,
 					   TRUE,TRUE,0);
 			GtkWidget *button=gtk_button_new_with_label(_("Browse"));
-			gtk_signal_connect(GTK_OBJECT(button),"clicked",
-					   GTK_SIGNAL_FUNC(my_gtk_aeditor_browse),editor);
+			g_signal_connect(G_OBJECT(button),"clicked",
+					 G_CALLBACK(my_gtk_aeditor_browse),editor);
 			gtk_box_pack_start(GTK_BOX(hbox),button,
 					   FALSE,FALSE,0);
 			gtk_container_add(GTK_CONTAINER(editor->frame),hbox);
@@ -450,8 +453,7 @@ static void aeditor_select_mode_int(MyGtkAEditor *editor,int i){
 		};
 		case SACT_SAVE_LIST:{
 			editor->path_entry=my_gtk_combo_new(ALL_HISTORIES[LOAD_SAVE_HISTORY]);//gtk_entry_new();
-			GtkWidget *hbox=gtk_hbox_new(FALSE,0);
-			gtk_box_set_spacing(GTK_BOX(hbox),5);
+			GtkWidget *hbox=gtk_hbox_new(FALSE,5);
 			editor->frame_child=hbox;
 			gtk_box_pack_start(GTK_BOX(hbox),
 					   gtk_label_new(_("Path:")),
@@ -459,8 +461,8 @@ static void aeditor_select_mode_int(MyGtkAEditor *editor,int i){
 			gtk_box_pack_start(GTK_BOX(hbox),editor->path_entry,
 					   TRUE,TRUE,0);
 			GtkWidget *button=gtk_button_new_with_label(_("Browse"));
-			gtk_signal_connect(GTK_OBJECT(button),"clicked",
-					   GTK_SIGNAL_FUNC(my_gtk_aeditor_browse),editor);
+			g_signal_connect(G_OBJECT(button),"clicked",
+					 G_CALLBACK(my_gtk_aeditor_browse),editor);
 			gtk_box_pack_start(GTK_BOX(hbox),button,
 					   FALSE,FALSE,0);
 			gtk_container_add(GTK_CONTAINER(editor->frame),hbox);
@@ -484,7 +486,9 @@ static void aeditor_select_mode_int(MyGtkAEditor *editor,int i){
 };
 
 static void aeditor_select_mode(GtkWidget *widget,MyGtkAEditor *editor){
-	GSList *group=gtk_radio_menu_item_group((GtkRadioMenuItem *)((GtkOptionMenu *)editor->omenu)->menu_item);
+	aeditor_select_mode_int(editor,gtk_combo_box_get_active(GTK_COMBO_BOX(widget)));
+/*	
+	GSList *group=gtk_radio_menu_item_get_group((GtkRadioMenuItem *)widget);
 	int i=0;
 	while(group && !((GtkCheckMenuItem *)(group->data))->active){
 		group = group->next;
@@ -492,12 +496,20 @@ static void aeditor_select_mode(GtkWidget *widget,MyGtkAEditor *editor){
 	};
 	i=SACT_LAST-i-1;
 	aeditor_select_mode_int(editor,i);
+*/
 };
 
 static GtkWidget *my_option_menu (char *labels[],
 				  gint num_items,
 				  gint active,
 				  gpointer data){
+	GtkWidget *combo_box = gtk_combo_box_new_text ();
+	for (int i=0;i<num_items;i++){
+		gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box), _(labels[i]));
+	};
+	g_signal_connect(G_OBJECT(combo_box),"changed",(GtkSignalFunc)aeditor_select_mode,data);
+	return(combo_box);
+/*	
 	GtkWidget *omenu;
 	GtkWidget *menu;
 	GtkWidget *menu_item;
@@ -511,12 +523,12 @@ static GtkWidget *my_option_menu (char *labels[],
 	
 	for (i = 0; i < num_items; i++){
 		menu_item = gtk_radio_menu_item_new_with_label (group, _(labels[i]));
-		gtk_signal_connect (GTK_OBJECT (menu_item),
-				    "activate",
-				    (GtkSignalFunc)aeditor_select_mode,
-				    data);
-		group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menu_item));
-		gtk_menu_append (GTK_MENU (menu), menu_item);
+		g_signal_connect(G_OBJECT (menu_item),
+				 "activate",
+				 (GtkSignalFunc)aeditor_select_mode,
+				 data);
+		group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menu_item));
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
 		if (i == active)
 			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), TRUE);
 		gtk_widget_show (menu_item);
@@ -526,22 +538,26 @@ static GtkWidget *my_option_menu (char *labels[],
 	gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), active);
 
 	return omenu;
+*/
 };
 
 static void my_gtk_aeditor_destroy(GtkObject *widget){
 	g_return_if_fail(widget!=NULL);
 	MyGtkAEditor *editor=MY_GTK_AEDITOR(widget);
-	if (editor->dwn) delete(editor->dwn);
+	if (editor->dwn){
+		delete(editor->dwn);
+		editor->dwn=NULL;
+	};
 	if (editor->action) editor->action->lock=0;
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (widget);
+	if (GTK_OBJECT_CLASS (parent_class2)->destroy)
+		(* GTK_OBJECT_CLASS (parent_class2)->destroy) (widget);
 };
 
 static void my_gtk_aeditor_class_init(MyGtkAEditorClass *klass){
 	GtkObjectClass *object_class=(GtkObjectClass *)klass;
 	
 	object_class->destroy=my_gtk_aeditor_destroy;
-	parent_class=(GtkWidgetClass *)gtk_type_class(gtk_window_get_type());
+	parent_class2=(GtkWidgetClass *)gtk_type_class(gtk_window_get_type());
 };
 
 static void my_gtk_aeditor_ok(GtkWidget *widget,MyGtkAEditor *editor){
@@ -613,19 +629,19 @@ static void my_gtk_aeditor_ok(GtkWidget *widget,MyGtkAEditor *editor){
 	};
 	case SACT_RUN_DOWNLOAD:{
 		d4xSARunDownload *act=new d4xSARunDownload;
-		act->url=new tAddr(text_from_combo(editor->url_entry));
+		act->url=std::string(text_from_combo(editor->url_entry));
 		action=act;
 		break;
 	};
 	case SACT_PAUSE_DOWNLOAD:{
 		d4xSAStopDownload *act=new d4xSAStopDownload;
-		act->url=new tAddr(text_from_combo(editor->url_entry));
+		act->url=std::string(text_from_combo(editor->url_entry));
 		action=act;
 		break;
 	};
 	case SACT_DELETE_DOWNLOAD:{
 		d4xSADelDownload *act=new d4xSADelDownload;
-		act->url=new tAddr(text_from_combo(editor->url_entry));
+		act->url=std::string(text_from_combo(editor->url_entry));
 		action=act;
 		break;
 	};
@@ -647,12 +663,11 @@ static void my_gtk_aeditor_ok(GtkWidget *widget,MyGtkAEditor *editor){
 			delete(act->dwn);
 		if (editor->dwn==NULL){
 			editor->dwn=new tDownload;
+			editor->dwn->config=new tCfg;
 			editor->dwn->set_default_cfg();
-			editor->dwn->config.save_path.set(CFG.GLOBAL_SAVE_PATH);
+			editor->dwn->config->save_path.set(CFG.GLOBAL_SAVE_PATH);
+			editor->dwn->info=std::string(text_from_combo(editor->url_entry));
 		};
-		if (editor->dwn->info)
-			delete(editor->dwn->info);
-		editor->dwn->info=new tAddr(text_from_combo(editor->url_entry));
 		act->dwn=editor->dwn;
 		editor->dwn=(tDownload*)NULL;
 		action=act;
@@ -660,7 +675,7 @@ static void my_gtk_aeditor_ok(GtkWidget *widget,MyGtkAEditor *editor){
 	};
 	case SACT_DEL_IF_COMPLETED:{
 		d4xSADelIfCompleted *act=new d4xSADelIfCompleted;
-		act->url=new tAddr(text_from_combo(editor->url_entry));
+		act->url=std::string(text_from_combo(editor->url_entry));
 		action=act;
 		break;
 	};
@@ -675,7 +690,15 @@ static void my_gtk_aeditor_ok(GtkWidget *widget,MyGtkAEditor *editor){
 };
 
 static void my_gtk_aeditor_cancel(GtkWidget *widget,GtkWidget *editor){
+	printf("my_gtk_aeditor_cancel\n");
 	gtk_widget_destroy(editor);
+	printf("/my_gtk_aeditor_cancel\n");
+};
+
+static void my_gtk_aeditor_close(GtkWidget *widget,GdkEvent *event,GtkWidget *editor){
+	printf("my_gtk_aeditor_close\n");
+	gtk_widget_destroy(editor);
+	printf("/my_gtk_aeditor_close\n");
 };
 
 static void my_gtk_aeditor_retry_cb(GtkWidget *widget,MyGtkAEditor *editor){
@@ -692,15 +715,13 @@ static void my_gtk_aeditor_init(MyGtkAEditor *editor){
 	editor->dwn=(tDownload *)NULL;
 	editor->action=(d4xSchedAction *)NULL;
 	editor->frame_child=(GtkWidget *)NULL;
-	editor->hbox=gtk_hbox_new(FALSE,0);
-	gtk_box_set_spacing(GTK_BOX(editor->hbox),5);
+	editor->hbox=gtk_hbox_new(FALSE,5);
 	gtk_box_set_spacing(GTK_BOX(vbox),5);
 	editor->omenu = my_option_menu (action_names,
 					sizeof(action_names)/sizeof(char*),
 					0, editor);
 
-	GtkWidget *hbox=gtk_hbox_new(FALSE,0);
-	gtk_box_set_spacing(GTK_BOX(hbox),1);
+	GtkWidget *hbox=gtk_hbox_new(FALSE,1);
 
 	GtkWidget *label=gtk_label_new(_("Start time(hour:min:sec)"));
 	editor->hour=my_gtk_entry_new_with_max_length(2,0);
@@ -716,11 +737,10 @@ static void my_gtk_aeditor_init(MyGtkAEditor *editor){
 	gtk_box_pack_start(GTK_BOX(hbox),editor->sec,FALSE,FALSE,0);
 
 
-	GtkWidget *tmpvbox=gtk_vbox_new(FALSE,0);
-	gtk_box_set_spacing(GTK_BOX(tmpvbox),5);
+	GtkWidget *tmpvbox=gtk_vbox_new(FALSE,5);
 
 	editor->calendar=gtk_calendar_new();
-	gtk_calendar_display_options(GTK_CALENDAR(editor->calendar),
+	gtk_calendar_set_display_options(GTK_CALENDAR(editor->calendar),
 				     GtkCalendarDisplayOptions(
 				     GTK_CALENDAR_WEEK_START_MONDAY |
 				     GTK_CALENDAR_SHOW_DAY_NAMES|
@@ -729,15 +749,13 @@ static void my_gtk_aeditor_init(MyGtkAEditor *editor){
 	gtk_box_pack_start(GTK_BOX(tmpvbox),editor->calendar,FALSE,FALSE,0);
 	gtk_box_pack_start(GTK_BOX(editor->hbox),tmpvbox,FALSE,FALSE,0);
 
-	tmpvbox=gtk_vbox_new(FALSE,0);
-	gtk_box_set_spacing(GTK_BOX(tmpvbox),5);
+	tmpvbox=gtk_vbox_new(FALSE,5);
 	editor->retry=gtk_check_button_new_with_label(_("Retry this action"));
-	gtk_signal_connect(GTK_OBJECT(editor->retry),"clicked",
-			   GTK_SIGNAL_FUNC(my_gtk_aeditor_retry_cb),editor);
+	g_signal_connect(G_OBJECT(editor->retry),"clicked",
+			 G_CALLBACK(my_gtk_aeditor_retry_cb),editor);
 	gtk_box_pack_start (GTK_BOX (tmpvbox),editor->retry,FALSE, FALSE, 0);
 
-	hbox=gtk_hbox_new(FALSE,0);
-	gtk_box_set_spacing(GTK_BOX(hbox),5);
+	hbox=gtk_hbox_new(FALSE,5);
 	editor->retry_times=my_gtk_entry_new_with_max_length(2,1);
 	gtk_box_pack_start (GTK_BOX (hbox),editor->retry_times,FALSE, FALSE, 0);
 	label=gtk_label_new(_("times to repeat (-1 unlimited)"));
@@ -745,8 +763,7 @@ static void my_gtk_aeditor_init(MyGtkAEditor *editor){
 	gtk_box_pack_start (GTK_BOX (tmpvbox),hbox,FALSE, FALSE, 0);
 
 	label=gtk_label_new(_("period (day/hour:min)"));
-	hbox=gtk_hbox_new(FALSE,0);
-	gtk_box_set_spacing(GTK_BOX(hbox),1);
+	hbox=gtk_hbox_new(FALSE,1);
 	editor->period_hours=my_gtk_entry_new_with_max_length(2,0);
 	editor->period_mins=my_gtk_entry_new_with_max_length(2,0);
 	editor->period_days=my_gtk_entry_new_with_max_length(2,0);
@@ -762,7 +779,7 @@ static void my_gtk_aeditor_init(MyGtkAEditor *editor){
 
 
 	GtkWidget *frame=gtk_frame_new(_("Action"));
-	gtk_container_border_width(GTK_CONTAINER(frame),5);
+	gtk_container_set_border_width(GTK_CONTAINER(frame),5);
 	gtk_container_add(GTK_CONTAINER(frame),editor->omenu);
 	gtk_box_pack_end(GTK_BOX (tmpvbox),frame,FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(editor->hbox),tmpvbox,FALSE,TRUE,0);
@@ -770,25 +787,28 @@ static void my_gtk_aeditor_init(MyGtkAEditor *editor){
 	gtk_box_pack_start(GTK_BOX(vbox),editor->hbox,FALSE,TRUE,0);
 	
 	editor->frame=gtk_frame_new(_("Properties"));
-	gtk_container_border_width(GTK_CONTAINER(editor->frame),5);
+	gtk_container_set_border_width(GTK_CONTAINER(editor->frame),5);
 	gtk_box_pack_start(GTK_BOX(vbox),editor->frame,FALSE,TRUE,0);
 	
 	hbox=gtk_hbutton_box_new();
 	gtk_box_set_spacing(GTK_BOX(hbox),5);
-	GtkWidget *ok_button=gtk_button_new_with_label(_("Ok"));
-	GtkWidget *cancel_button=gtk_button_new_with_label(_("Cancel"));
+	GtkWidget *ok_button=gtk_button_new_from_stock(GTK_STOCK_OK);
+	GtkWidget *cancel_button=gtk_button_new_from_stock(GTK_STOCK_CANCEL);
 	gtk_box_pack_end(GTK_BOX(hbox),ok_button,FALSE,FALSE,0);
 	gtk_box_pack_end(GTK_BOX(hbox),cancel_button,FALSE,FALSE,0);
-	gtk_signal_connect(GTK_OBJECT(ok_button),"clicked",
-			   GTK_SIGNAL_FUNC(my_gtk_aeditor_ok),editor);
-	gtk_signal_connect(GTK_OBJECT(cancel_button),"clicked",
-			   GTK_SIGNAL_FUNC(my_gtk_aeditor_cancel),editor);
+	g_signal_connect(G_OBJECT(ok_button),"clicked",
+			 G_CALLBACK(my_gtk_aeditor_ok),editor);
+	g_signal_connect(G_OBJECT(cancel_button),"clicked",
+			 G_CALLBACK(my_gtk_aeditor_cancel),editor);
+	g_signal_connect(G_OBJECT(editor),"delete_event",
+			 G_CALLBACK(my_gtk_aeditor_close), editor);
+	d4x_eschandler_init(GTK_WIDGET(editor),editor);
 	gtk_box_pack_start (GTK_BOX (vbox),hbox,
 			    FALSE, TRUE, 0);
 
 	gtk_window_set_title(GTK_WINDOW(editor),_("Edit scheduled action"));
 	gtk_window_set_position(GTK_WINDOW(editor),GTK_WIN_POS_CENTER);
-	gtk_container_border_width(GTK_CONTAINER(editor),5);
+	gtk_container_set_border_width(GTK_CONTAINER(editor),5);
 	gtk_container_add(GTK_CONTAINER(editor),vbox);
 
 };
@@ -796,24 +816,25 @@ static void my_gtk_aeditor_init(MyGtkAEditor *editor){
 guint my_gtk_aeditor_get_type(){
 	static guint my_aeditor_type=0;
 	if (!my_aeditor_type){
-		GtkTypeInfo my_aeditor_info={
-			"MyGtkAEditor",
-			sizeof(MyGtkAEditor),
+		GTypeInfo my_aeditor_info={
 			sizeof(MyGtkAEditorClass),
-			(GtkClassInitFunc)my_gtk_aeditor_class_init,
-			(GtkObjectInitFunc)my_gtk_aeditor_init,
-			NULL,NULL
-//			(GtkArgSetFunc) NULL,
-//			(GtkArgGetFunc) NULL
+			NULL,
+			NULL,
+			(GClassInitFunc)my_gtk_aeditor_class_init,
+			NULL,
+			NULL,
+			sizeof(MyGtkAEditor),
+			0,
+			(GInstanceInitFunc)my_gtk_aeditor_init
 		};
-		my_aeditor_type = gtk_type_unique (gtk_window_get_type (), &my_aeditor_info);
+		my_aeditor_type =g_type_register_static (GTK_TYPE_WINDOW,"MyGtkAEditor",&my_aeditor_info,(GTypeFlags)0);
 	};
 	return my_aeditor_type;
 };
 
 
-GtkWidget *my_gtk_aeditor_new(d4xSchedAction *action=(d4xSchedAction *)NULL){
-	MyGtkAEditor *editor=(MyGtkAEditor *)gtk_type_new(my_gtk_aeditor_get_type());
+GtkWidget *my_gtk_aeditor_new(d4xSchedAction *action){
+	MyGtkAEditor *editor=(MyGtkAEditor *)g_object_new(my_gtk_aeditor_get_type(),NULL);
 	char data[MAX_LEN];
 	if (action){
 		struct tm date;
@@ -844,7 +865,8 @@ GtkWidget *my_gtk_aeditor_new(d4xSchedAction *action=(d4xSchedAction *)NULL){
 		};
 		
 		editor->action=action;
-		gtk_option_menu_set_history (GTK_OPTION_MENU (editor->omenu),action->type());
+//		gtk_option_menu_set_history (GTK_OPTION_MENU (editor->omenu),action->type());
+		gtk_combo_box_set_active(GTK_COMBO_BOX(editor->omenu),action->type());
 		aeditor_select_mode_int(editor,action->type());
 	}else{
 		time_t now=time((time_t*)NULL);
@@ -858,14 +880,15 @@ GtkWidget *my_gtk_aeditor_new(d4xSchedAction *action=(d4xSchedAction *)NULL){
 		text_to_combo(editor->sec,data);
 		gtk_calendar_select_month(GTK_CALENDAR(editor->calendar),date.tm_mon,date.tm_year+1900);
 		gtk_calendar_select_day(GTK_CALENDAR(editor->calendar),date.tm_mday);
-		aeditor_select_mode((GtkWidget *)NULL,editor);
+		gtk_combo_box_set_active(GTK_COMBO_BOX(editor->omenu),0);
+//		aeditor_select_mode_int(editor,0);
 		if (editor->action && editor->action->type()==SACT_SET_SPEED){
 			d4xSASpeed *act=(d4xSASpeed *)(editor->action);
 			if (act->speed==1)
-				gtk_signal_emit_by_name(GTK_OBJECT(editor->sb_low), "clicked");
+				g_signal_emit_by_name(G_OBJECT(editor->sb_low),"clicked");
 			else
-				gtk_signal_emit_by_name(act->speed==2?GTK_OBJECT(editor->sb_middle):GTK_OBJECT(editor->sb_high),
-							"clicked");
+				g_signal_emit_by_name(act->speed==2?G_OBJECT(editor->sb_middle):G_OBJECT(editor->sb_high),
+						      "clicked");
 		};
 	};
 	if (!GTK_TOGGLE_BUTTON(editor->retry)->active){
