@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -18,13 +18,18 @@ enum FS_FACE_COLUMNS{
 	FS_COL_ICON,
 	FS_COL_NAME,
 	FS_COL_SIZE,
+	FS_COL_COUNT,
 	FS_COL_LAST
 };
 
-void fs_list_remove(GtkCList *clist,tDownload *what);
-void fs_list_add(GtkCList* clist,tDownload *what);
-void fs_list_set_icon(GtkCList *clist,tDownload *what,int icon);
-GtkCList *fs_list_init();
+//extern tDownload *FS_CUR_SELECTED;
+
+void fs_list_remove(GtkTreeView *view,tDownload *what);
+void fs_list_add(GtkTreeView *view,tDownload *what);
+void fs_list_set_icon(GtkTreeView *view,tDownload *what,int icon);
+void fs_list_set_count(GtkTreeView *view,tDownload *what);
+GtkTreeView *fs_list_init();
+GtkTreeView *fs_list_init_sublist();
 void fs_list_set_size();
 void fs_list_get_size();
 void fs_list_hide();
