@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -17,6 +17,7 @@ GtkWidget *make_menu_item(char *name,char *accel,GdkPixmap *pixmap,GdkBitmap *bi
 void init_list_menu();
 void list_menu_prepare();
 void lm_inv_protect_flag();
+void lm_open_file();
 
 enum {
 	LM_LOG,
@@ -29,9 +30,11 @@ enum {
 	LM_DEL,
 	LM_MOVEUP,
 	LM_MOVEDOWN,
-	LM_SET_LIMIT,
 	LM_DELC,
 	LM_DELF,
+	LM_OPENFOLDER,
+	LM_OPENFILE,
+	LM_ALT,
 	LM_SEARCH,
 	LM_LAST
 };
