@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -8,6 +8,7 @@
  *	but WITHOUT ANY WARRANTY; without even the implied warranty of
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
+
 #include "html.h"
 #include "locstr.h"
 #include "var.h"
@@ -230,7 +231,7 @@ tHtmlTagField::~tHtmlTagField(){
 };
 
 tHtmlTag::tHtmlTag(){
-	name=NULL;
+	descr=name=NULL;
 	fields=NULL;
 };
 
@@ -238,20 +239,31 @@ void tHtmlTag::print(){
 	printf("Name:\t%s\n",name==NULL?"*NULL*":name);
 };
 
+tHtmlTagField *tHtmlTag::find_field(const char *n){
+	tHtmlTagField *fld=(tHtmlTagField *)(fields->first());
+	while(fld){
+		if (strcasecmp(fld->name,n)==0)
+			return fld;
+		fld=(tHtmlTagField *)(fields->prev());
+	};
+	return 0;
+};
+
 tHtmlTag::~tHtmlTag(){
 	if (name) delete[] name;
+	if (descr) delete[] descr;
 	if (fields) delete(fields);
 };
 /********************************************************/
 tHtmlUrl::tHtmlUrl(){
-	info=NULL;
+	descr=NULL;
 };
 
 void tHtmlUrl::print(){
 };
 
 tHtmlUrl::~tHtmlUrl(){
-	if (info) delete(info);
+	if (descr) delete[] descr;
 };
 
 
@@ -415,6 +427,30 @@ char *tHtmlParser::extract_from_icommas(char *str){
 	return(copy_string(temp));
 };
 
+void tHtmlParser::get_tag_descr(tHtmlTag *tag){
+	if (strcasecmp(tag->name,"img")==0){
+		//get descr from ALT or TITLE field
+		tHtmlTagField *fld=tag->find_field("alt");
+		if (!fld)
+			fld=tag->find_field("title");
+		if (fld) tag->descr=copy_string(fld->value);
+	}else if (strcasecmp(tag->name,"a")==0){
+		//get descr from content of tag
+		char *str=new char[51];
+		char *p=str;
+		while(p-str<50 && WL->read(p,sizeof(char))>0){
+			if (*p=='<'){
+				WL->shift(-1,SEEK_CUR);
+				break;
+			};
+			p++;
+		};
+		*p=0;
+		WL->shift(str-p,SEEK_CUR);
+		tag->descr=str;
+	};
+};
+
 tHtmlTag *tHtmlParser::get_tag(){
 	char p;
 	tHtmlTag *rvalue=NULL;
@@ -432,10 +468,12 @@ tHtmlTag *tHtmlParser::get_tag(){
 						if (p=='>') break;
 					};
 					rvalue=get_tag();
+					delete[] name;
 				}else{
 					rvalue=new tHtmlTag;
 					rvalue->name=name;
 					get_fields(rvalue);
+					get_tag_descr(rvalue);
 				};
 			};
 			break;
@@ -446,7 +484,7 @@ tHtmlTag *tHtmlParser::get_tag(){
 	return rvalue;
 };
 
-static void write_up_dirs(int out_fd,char *a){
+static void write_up_dirs(int out_fd,const char *a){
 	int depth=0;
 	while(*a){
 		depth=1;
@@ -460,12 +498,11 @@ static void write_up_dirs(int out_fd,char *a){
 	};
 };
 
-tAddr *fix_url_global(char *url,tAddr *papa,int out_fd,int leave){
-	if (url==NULL || *url==0) return(NULL);
-	tAddr *info=NULL;
+d4x::URL fix_url_global(char *url,const d4x::URL &papa,int out_fd,int leave,int quest_sign_replace){
+	if (url==NULL || *url==0) return d4x::URL();
+	d4x::URL info;
 	char *html_shift=NULL;
 	if (!global_url(url)) {
-		info=new tAddr;
 		html_shift=rindex(url,'#');
 		if (html_shift){
 			*html_shift=0;
@@ -473,10 +510,10 @@ tAddr *fix_url_global(char *url,tAddr *papa,int out_fd,int leave){
 		};
 		char *quest=index(url,'?');
 		if (quest){
-			info->params.set(quest+1);
+			info.params=(quest+1);
 			*quest=0;
 		};
-		if (papa->proto==D_PROTO_FTP){
+		if (papa.proto==D_PROTO_FTP){
 			quest=index(url,';');
 			if (quest)
 				*quest=0;
@@ -485,68 +522,69 @@ tAddr *fix_url_global(char *url,tAddr *papa,int out_fd,int leave){
 		char *tmp=parse_percents(url);
 		quest=rindex(tmp,'/');
 		if (quest) {
-			info->file.set(quest+1);
+			info.file=(quest+1);
 			*quest=0;
 			if (*tmp=='/')
-				info->path.set(tmp+1);
+				info.path=(tmp+1);
 			else{
 				if (tmp==quest)
-					info->path.set("");
+					info.path=("");
 				else
-					info->compose_path(papa->path.get(),tmp);
+					info.path=papa.path/std::string(tmp);
 			};
 			*quest='/';
 		} else {
-			info->path.set(papa->path.get());
-			info->file.set(tmp);
+			info.path=papa.path;
+			info.file=tmp;
 		};
 		delete[] tmp;
-		info->file_del_sq();
-		info->copy_host(papa);
+		info.file=info.file.substr(0,info.file.find('#'));
+		info.copy_host(papa);
 	}else{
 		if (begin_string_uncase(url,"http:") ||
 		    begin_string_uncase(url,"ftp:") ||
 		    begin_string_uncase(url,"https:")){
 			html_shift=rindex(url,'#');
 			if (html_shift)	html_shift+=1;
-			info=new tAddr(url);
-			if (papa->proto==D_PROTO_FTP && info->proto==D_PROTO_FTP){
-				char *quest=index(info->file.get(),';');
-				if (quest)
-					*quest=0;
+			info=std::string(url);
+			if (papa.proto==D_PROTO_FTP && info.proto==D_PROTO_FTP){
+				info.file=info.file.substr(0,info.file.find(';'));
 			};
 		}else{
 			if (out_fd>=0){
 				f_wstr(out_fd,url);
 			};
-			return(NULL);
+			return d4x::URL();
 		};
 	};
 	if (out_fd>=0){
-		if (!equal(info->host.get(),papa->host.get())){
+		if (info.host!=papa.host){
 			if (leave){
 				f_wstr(out_fd,"../");
-				write_up_dirs(out_fd,papa->path.get());
-				f_wstr(out_fd,info->host.get());
+				write_up_dirs(out_fd,papa.path.c_str());
+				f_wstr(out_fd,info.host.c_str());
 				f_wchar(out_fd,'/');
-				f_wstr(out_fd,info->path.get());
+				f_wstr(out_fd,info.path.c_str());
 				f_wchar(out_fd,'/');
-				f_wstr(out_fd,info->file.get());
-				if (info->params.get()){
-					f_wstr(out_fd,"%3f");
-					f_wstr(out_fd,info->params.get());
+				if (quest_sign_replace && info.file.c_str()[0]==0)
+					f_wstr(out_fd,"index_html");
+				else
+					f_wstr(out_fd,info.file.c_str());
+				if (!info.params.empty()){
+					f_wstr(out_fd,quest_sign_replace?"%5f":"%3f");
+					f_wstr(out_fd,info.params.c_str());
 				};
 			}else{
-				char *url=info->url();
-				f_wstr(out_fd,url);
-				delete[] url;
+				f_wstr(out_fd,std::string(info).c_str());
 			};
 		}else{
-			char *a=papa->path.get();
-			char *b=info->path.get();
+			//FIXME: next code is too ugly :-)
+			
+			const char *a=papa.path.c_str(); 
+			const char *b=info.path.c_str();
 			if (*a=='/') a+=1;
 			if (*b=='/') b+=1;
-			char *l=b;
+			const char *l=b;
 			while(*a){
 				if (*a==*b){
 					if (*b=='/')
@@ -566,10 +604,13 @@ tAddr *fix_url_global(char *url,tAddr *papa,int out_fd,int leave){
 				f_wstr(out_fd,l);
 				f_wchar(out_fd,'/');
 			};
-			f_wstr(out_fd,info->file.get());
-			if (info->params.get()){
-				f_wstr(out_fd,"%3f");
-				f_wstr(out_fd,info->params.get());
+			if (quest_sign_replace && info.file.empty())
+				f_wstr(out_fd,"index.html");
+			else
+				f_wstr(out_fd,info.file.c_str());
+			if (!info.params.empty()){
+				f_wstr(out_fd,quest_sign_replace?"%5f":"%3f");
+				f_wstr(out_fd,info.params.c_str());
 			};
 		};
 		if (html_shift){
@@ -580,14 +621,26 @@ tAddr *fix_url_global(char *url,tAddr *papa,int out_fd,int leave){
 	return(info);
 };
 
-void tHtmlParser::fix_url(char *url,tQueue *list,tAddr *papa,const char *tag){
+char *tHtmlParser::convert_to_utf8(const char *src){
+	if (!src) return 0;
+	if((g_utf8_validate(src, -1, NULL)) == TRUE)
+		return copy_string(src);
+	//to avoid different memory freing schemes
+	char *tmp=g_convert(src,-1,"UTF-8",codepage.c_str(),NULL,NULL,NULL);
+	char *rval=copy_string(tmp);
+	g_free(tmp);
+	return rval;
+};
+
+void tHtmlParser::fix_url(char *url,tQueue *list,const d4x::URL &papa,const char *tag,const char *descr){
 	if (out_fd>=0) f_wstr(out_fd,"=\"");
-	tAddr *info=fix_url_global(url,papa,out_fd,leave);
+	d4x::URL info=fix_url_global(url,papa,out_fd,leave,quest_sign_replace);
 	if (out_fd>=0) f_wchar(out_fd,'\"');
-	if (info){
+	if (info.is_valid()){
 		tHtmlUrl *node=new tHtmlUrl;
-		info->tag.set(tag);
+		info.tag=tag;
 		node->info=info;
+		node->descr=convert_to_utf8(descr);
 		list->insert(node);
 	};
 };
@@ -613,7 +666,7 @@ void tHtmlParser::write_left_fields(tHtmlTag *tag){
 
 void tHtmlParser::look_for_meta_content(tHtmlTagField *where,
 					tQueue *list,
-					tAddr *papa,
+					const d4x::URL &papa,
 					const char *tag){
 	tHtmlTagField *field=(tHtmlTagField *)(where->prev);
 	while (field){
@@ -645,7 +698,23 @@ void tHtmlParser::look_for_meta_content(tHtmlTagField *where,
 	};
 };
 
-void tHtmlParser::parse(tWriterLoger *wl,tQueue *list,tAddr *papa){
+void tHtmlParser::set_content_type(const char *ct){
+	//Example: text/html; charset=koi8-r
+	char *a=index(ct,'=');
+	if (a) codepage=a+1;
+};
+
+
+void tHtmlParser::get_charset_from_meta(tHtmlTagField *fld){
+	if (fld && fld->value){
+		char *val=extract_from_icommas(fld->value);
+		set_content_type(val);
+		delete [] val;
+	};
+};
+
+void tHtmlParser::parse(tWriterLoger *wl,tQueue *list,const d4x::URL &papa,int qsignreplace){
+	quest_sign_replace=qsignreplace;
 	list->done();
 	list->init(0);
 	base=NULL;
@@ -685,6 +754,8 @@ void tHtmlParser::parse(tWriterLoger *wl,tQueue *list,tAddr *papa){
 						case HF_TYPE_META:
 							if (tmp && equal_uncase(tmp,"refresh"))
 								look_for_meta_content(field,list,papa,temp->name);
+							if (tmp && equal_uncase(tmp,"Content-Type"))
+								get_charset_from_meta(temp->find_field("content"));
 							break;
 						case HF_TYPE_BASE:
 							if (base) delete[] base;
@@ -705,7 +776,7 @@ void tHtmlParser::parse(tWriterLoger *wl,tQueue *list,tAddr *papa){
 									f_wchar(out_fd,' ');
 									f_wstr(out_fd,field->name);
 								};
-								fix_url(tmp,list,papa,temp->name);
+								fix_url(tmp,list,papa,temp->name,temp->descr);
 							};
 						};
 						delete[] tmp;
