@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -14,6 +14,12 @@
 #include "queue.h"
 #include <pthread.h>
 #include "log.h"
+#include "mutex.h"
+
+enum MSG_QUEUE_TYPES{
+	MQT_MY=1,
+	MQT_COM=2
+};
 
 struct tLogMsg:public tNode{
 	long type;
@@ -23,11 +29,12 @@ struct tLogMsg:public tNode{
 };
 
 class tMsgQueue:public tQueue{
-	pthread_mutex_t mylock;
+	d4x::Mutex mylock;
  public:
 	tMsgQueue();
 	void insert(tNode *what);
     	void insert_before(tNode *what,tNode *where);
+	void remove_this_log(tLog *log);
 	void del(tNode *what);
 	~tMsgQueue();
 };
