@@ -50,7 +50,7 @@ void d4xPing::run(tDList *list,tWriterLoger *WL){
 		data[TOTAL].ref=NULL;
 		tmp->status=0;
 		tmp->action=0;
-		tmp->Attempt.curent+=1; // Used for calculation number of attempts
+		tmp->Attempt=1+tmp->Attempt; // Used for calculation number of attempts
 		int len=my_get_host_by_name(tmp->info.host.c_str(),tmp->info.port,
 					    &info,&hp,
 					    buf,MAX_LEN,&rval);
