@@ -21,6 +21,7 @@ struct tSavedVar{
 enum SV_TYPES{
 	SV_TYPE_INT,
 	SV_TYPE_LINT,
+	SV_TYPE_FSIZE_TRIGER,
 	SV_TYPE_PSTR,
 	SV_TYPE_STDSTR,
 	SV_TYPE_SPLIT,
@@ -28,8 +29,6 @@ enum SV_TYPES{
 	SV_TYPE_URL,
 	SV_TYPE_TIME,
 	SV_TYPE_DOWNLOAD,
-	SV_TYPE_RULE,
-	SV_TYPE_FILTER,
 	SV_TYPE_QDOWNLOAD,
 	SV_TYPE_QUEUE,
 	SV_TYPE_QV,
