@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -17,5 +17,7 @@ void save_list();
 void read_list();
 int save_list_to_file(char *path);
 int read_list_from_file(char *path);
+int save_list_to_file_current(char *path);
+int read_list_from_file_current(char *path);
 
 #endif
