@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -12,11 +12,12 @@
 #ifndef __DOWNLOADER_SEGMENTS_HEADER__
 #define __DOWNLOADER_SEGMENTS_HEADER__
 
-#include <pthread.h>
+#include "mutex.h"
 #include "queue.h"
+typedef unsigned long long segoff_t;
 
 struct tSegment{
-	unsigned long int begin,end;
+	segoff_t  begin,end;
 	long offset_in_file;
 	tSegment *next,*prev;
 	tSegment();
@@ -26,19 +27,16 @@ struct tSegment{
 };
 
 class tSegmentator{
+	int autosave_counter;
 	tSegment *FIRST,*LAST;
 	tSegment *HEAP;
-	pthread_mutex_t lockmutex;
+	d4x::Mutex lockmutex;
 	int fd;
 	char *filename;
-	unsigned long int total;
+	segoff_t total;
 	int load();
-	int save();
 	void lock();
 	void unlock();
-	int join(tSegment *what);
-	void remove(tSegment *what);
-	void save_from(tSegment *what);
 	tSegment *seg_alloc();
 	void seg_free(tSegment *seg);
  public:
@@ -46,12 +44,14 @@ class tSegmentator{
 	tSegmentator(char *path);
 	void init(char *path);
 	void print();
-	void insert(unsigned long int begin, unsigned long int end);
-	void truncate(unsigned long int shift);
+	int insert(segoff_t begin, segoff_t end);
+	void truncate(segoff_t shift);
 	tSegment *get_first();
-	tSegment *to_holes(unsigned long int size);
-	unsigned long int get_total();
+	tSegment *to_holes(segoff_t size);
+	int one_segment();
+	segoff_t get_total();
 	void done();
+	int save();
 	void complete();
 	void lock_public();
 	void unlock_public();
