@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -20,6 +20,19 @@ void my_pthread_key_init();
 tDownload **my_pthread_key_get();
 void my_pthread_key_set(tDownload *what);
 void my_pthreads_mutex_init(pthread_mutex_t *lock);
-void download_set_block(int a);
+
+namespace d4x{
+	class USR1Off2On{
+	public:
+		USR1Off2On();
+		~USR1Off2On();
+	};
+	
+	class USR1On2Off{
+	public:
+		USR1On2Off();
+		~USR1On2Off();
+	};
+};
 
 #endif
